@@ -1,4 +1,3 @@
-#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -14,20 +13,18 @@
 import uuid
 
 import mock
-from oslo_utils import reflection
-import six
 
 from heat.common import exception
 from heat.common import template_format
 from heat.engine import resource
 from heat.engine import scheduler
-from heat.tests import common
+from heat.tests.common import HeatTestCase
 from heat.tests import utils
 
 from ..resources import cloudnetworks  # noqa
 
 try:
-    from pyrax.exceptions import NotFound  # noqa
+    from pyrax.exceptions import NotFound
 except ImportError:
     from ..resources.cloudnetworks import NotFound  # noqa
 
@@ -75,17 +72,8 @@ class FakeClient(object):
             pass
 
 
-class FakeClientRaiseException(FakeClient):
-
-    def create(self, label=None, cidr=None):
-        raise Exception
-
-    def get(self, nwid):
-        raise Exception
-
-
 @mock.patch.object(cloudnetworks.CloudNetwork, "cloud_networks")
-class CloudNetworkTest(common.HeatTestCase):
+class CloudNetworkTest(HeatTestCase):
 
     _template = template_format.parse("""
     heat_template_version: 2013-05-23
@@ -100,13 +88,13 @@ class CloudNetworkTest(common.HeatTestCase):
 
     def setUp(self):
         super(CloudNetworkTest, self).setUp()
+        utils.setup_dummy_db()
         resource._register_class("Rackspace::Cloud::Network",
                                  cloudnetworks.CloudNetwork)
 
     def _parse_stack(self):
-        class_name = reflection.get_class_name(self, fully_qualified=False)
         self.stack = utils.parse_stack(self._template,
-                                       stack_name=class_name)
+                                       stack_name=self.__class__.__name__)
 
     def _setup_stack(self, mock_client, *args):
         self.fake_cnw = FakeClient(*args)
@@ -118,6 +106,7 @@ class CloudNetworkTest(common.HeatTestCase):
         res = self.stack['cnw']
         self.assertEqual((res.CREATE, res.COMPLETE), res.state)
 
+    @utils.stack_delete_after
     def test_attributes(self, mock_client):
         self._setup_stack(mock_client)
         res = self.stack['cnw']
@@ -127,28 +116,15 @@ class CloudNetworkTest(common.HeatTestCase):
         self.assertEqual(expect_label, res.FnGetAtt('label'))
         self.assertEqual(expect_cidr, res.FnGetAtt('cidr'))
 
-    def test_create_bad_cidr(self, mock_client):
-        prop = self._template['resources']['cnw']['properties']
-        prop['cidr'] = "bad cidr"
+    @utils.stack_delete_after
+    def test_create_bad_cider(self, mock_client):
+        self._template['resources']['cnw']['properties']['cidr'] = "bad cidr"
         self._parse_stack()
         exc = self.assertRaises(exception.StackValidationFailed,
                                 self.stack.validate)
-        self.assertIn("Invalid net cidr", six.text_type(exc))
-        # reset property
-        prop['cidr'] = "172.16.0.0/24"
-
-    def test_check(self, mock_client):
-        self._setup_stack(mock_client)
-        res = self.stack['cnw']
-        scheduler.TaskRunner(res.check)()
-        self.assertEqual((res.CHECK, res.COMPLETE), res.state)
-
-        self.fake_cnw.networks = []
-        exc = self.assertRaises(exception.ResourceFailure,
-                                scheduler.TaskRunner(res.check))
-        self.assertEqual((res.CHECK, res.FAILED), res.state)
-        self.assertIn('No network', str(exc))
+        self.assertIn("Invalid cidr", str(exc))
 
+    @utils.stack_delete_after
     def test_delete(self, mock_client):
         self._setup_stack(mock_client)
         res = self.stack['cnw']
@@ -156,41 +132,15 @@ class CloudNetworkTest(common.HeatTestCase):
         scheduler.TaskRunner(res.delete)()
         self.assertEqual((res.DELETE, res.COMPLETE), res.state)
         exc = self.assertRaises(NotFound, self.fake_cnw.get, res_id)
-        self.assertIn(res_id, six.text_type(exc))
-
-    def test_delete_no_network_created(self, mock_client):
-        self.fake_cnw = FakeClientRaiseException()
-        mock_client.return_value = self.fake_cnw
-        self._parse_stack()
-        self.stack.create()
-        self.assertEqual((self.stack.CREATE, self.stack.FAILED),
-                         self.stack.state)
-        res = self.stack['cnw']
-        self.assertEqual((res.CREATE, res.FAILED), res.state)
-        scheduler.TaskRunner(res.delete)()
-        self.assertEqual((res.DELETE, res.COMPLETE), res.state)
-
-    def test_delete_in_use(self, mock_client):
-        self._setup_stack(mock_client)
-        res = self.stack['cnw']
-        fake_network = res.network()
-        fake_network.delete = mock.Mock()
-        fake_network.delete.side_effect = [cloudnetworks.NetworkInUse(), True]
-        mock_client.return_value = fake_network
-        fake_network.get = mock.Mock()
-        fake_network.get.side_effect = [cloudnetworks.NotFound()]
-
-        scheduler.TaskRunner(res.delete)()
-        self.assertEqual((res.DELETE, res.COMPLETE), res.state)
+        self.assertIn(res_id, str(exc))
 
+    @utils.stack_delete_after
     def test_delete_not_complete(self, mock_client):
         self._setup_stack(mock_client)
         res = self.stack['cnw']
-        mock_client.get = mock.Mock()
-
-        task = res.handle_delete()
-        self.assertFalse(res.check_delete_complete(task))
+        self.assertFalse(res.check_delete_complete(res.network()))
 
+    @utils.stack_delete_after
     def test_delete_not_found(self, mock_client):
         self._setup_stack(mock_client)
         self.fake_cnw.networks = []
