@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,22 +12,22 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from heat.api.aws import exception
+from heat.common import identifier
 from heat.common import wsgi
 from heat.rpc import client as rpc_client
-from heat.common import identifier
-from heat.api.aws import exception
 
 
 class SignalController(object):
     def __init__(self, options):
         self.options = options
-        self.engine = rpc_client.EngineClient()
+        self.rpc_client = rpc_client.EngineClient()
 
     def update_waitcondition(self, req, body, arn):
         con = req.context
         identity = identifier.ResourceIdentifier.from_arn(arn)
         try:
-            md = self.engine.metadata_update(
+            md = self.rpc_client.metadata_update(
                 con,
                 stack_identity=dict(identity.stack()),
                 resource_name=identity.resource_name,
@@ -42,7 +41,7 @@ class SignalController(object):
         con = req.context
         identity = identifier.ResourceIdentifier.from_arn(arn)
         try:
-            md = self.engine.resource_signal(
+            self.rpc_client.resource_signal(
                 con,
                 stack_identity=dict(identity.stack()),
                 resource_name=identity.resource_name,
