@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -18,14 +17,14 @@ endpoint for heat AWS-compatible CloudWatch API
 """
 from heat.api.aws import exception
 from heat.api.aws import utils as api_utils
-from heat.common import wsgi
-from heat.common import policy
 from heat.common import exception as heat_exception
-from heat.rpc import client as rpc_client
-from heat.rpc import api as engine_api
-
-import heat.openstack.common.rpc.common as rpc_common
+from heat.common import policy
+from heat.common import wsgi
+from heat.openstack.common.gettextutils import _
 from heat.openstack.common import log as logging
+from heat.openstack.common.rpc import common as rpc_common
+from heat.rpc import api as engine_api
+from heat.rpc import client as rpc_client
 
 logger = logging.getLogger(__name__)
 
@@ -39,23 +38,23 @@ class WatchController(object):
 
     def __init__(self, options):
         self.options = options
-        self.engine_rpcapi = rpc_client.EngineClient()
+        self.rpc_client = rpc_client.EngineClient()
         self.policy = policy.Enforcer(scope='cloudwatch')
 
     def _enforce(self, req, action):
         """Authorize an action against the policy.json."""
         try:
-            self.policy.enforce(req.context, action, {})
+            self.policy.enforce(req.context, action)
         except heat_exception.Forbidden:
-            raise exception.HeatAccessDeniedError("Action %s not allowed " %
-                                                  action + "for user")
-        except Exception as ex:
+            msg = _("Action %s not allowed for user") % action
+            raise exception.HeatAccessDeniedError(msg)
+        except Exception:
             # We expect policy.enforce to either pass or raise Forbidden
             # however, if anything else happens, we want to raise
             # HeatInternalFailureError, failure to do this results in
             # the user getting a big stacktrace spew as an API response
-            raise exception.HeatInternalFailureError("Error authorizing " +
-                                                     "action %s" % action)
+            msg = _("Error authorizing action %s") % action
+            raise exception.HeatInternalFailureError(msg)
 
     @staticmethod
     def _reformat_dimensions(dims):
@@ -138,7 +137,7 @@ class WatchController(object):
             name = None
 
         try:
-            watch_list = self.engine_rpcapi.show_watch(con, watch_name=name)
+            watch_list = self.rpc_client.show_watch(con, watch_name=name)
         except rpc_common.RemoteError as ex:
             return exception.map_remote_error(ex)
 
@@ -213,7 +212,7 @@ class WatchController(object):
                         # Filter criteria not met, return None
                         return
                 except KeyError:
-                    logger.warning("Invalid filter key %s, ignoring" % f)
+                    logger.warning(_("Invalid filter key %s, ignoring") % f)
 
             return result
 
@@ -222,15 +221,15 @@ class WatchController(object):
         # FIXME : Don't yet handle filtering by Dimensions
         filter_result = dict((k, v) for (k, v) in parms.iteritems() if k in
                              ("MetricName", "Namespace"))
-        logger.debug("filter parameters : %s" % filter_result)
+        logger.debug(_("filter parameters : %s") % filter_result)
 
         try:
             # Engine does not currently support query by namespace/metric
             # so we pass None/None and do any filtering locally
             null_kwargs = {'metric_namespace': None,
                            'metric_name': None}
-            watch_data = self.engine_rpcapi.show_watch_metric(con,
-                                                              **null_kwargs)
+            watch_data = self.rpc_client.show_watch_metric(con,
+                                                           **null_kwargs)
         except rpc_common.RemoteError as ex:
             return exception.map_remote_error(ex)
 
@@ -268,7 +267,7 @@ class WatchController(object):
         # need to process (each dict) for dimensions
         metric_data = api_utils.extract_param_list(parms, prefix='MetricData')
         if not len(metric_data):
-            logger.error("Request does not contain required MetricData")
+            logger.error(_("Request does not contain required MetricData"))
             return exception.HeatMissingParameterError("MetricData list")
 
         watch_name = None
@@ -293,7 +292,7 @@ class WatchController(object):
                     'Dimensions': dimensions}}
 
         try:
-            self.engine_rpcapi.create_watch_data(con, watch_name, data)
+            self.rpc_client.create_watch_data(con, watch_name, data)
         except rpc_common.RemoteError as ex:
             return exception.map_remote_error(ex)
 
@@ -319,24 +318,18 @@ class WatchController(object):
         state = api_utils.get_param_value(parms, 'StateValue')
 
         if state not in state_map:
-            logger.error("Invalid state %s, expecting one of %s" %
-                         (state, state_map.keys()))
-            return exception.HeatInvalidParameterValueError("Invalid state %s"
-                                                            % state)
+            msg = _('Invalid state %(state)s, '
+                    'expecting one of %(expect)s') % {
+                        'state': state,
+                        'expect': state_map.keys()}
+            logger.error(msg)
+            return exception.HeatInvalidParameterValueError(msg)
 
-        # Check for optional parameters
-        # FIXME : We don't actually do anything with these in the engine yet..
-        state_reason = None
-        state_reason_data = None
-        if 'StateReason' in parms:
-            state_reason = parms['StateReason']
-        if 'StateReasonData' in parms:
-            state_reason_data = parms['StateReasonData']
-
-        logger.debug("setting %s to %s" % (name, state_map[state]))
+        logger.debug(_("setting %(name)s to %(state)s") % {
+                     'name': name, 'state': state_map[state]})
         try:
-            self.engine_rpcapi.set_watch_state(con, watch_name=name,
-                                               state=state_map[state])
+            self.rpc_client.set_watch_state(con, watch_name=name,
+                                            state=state_map[state])
         except rpc_common.RemoteError as ex:
             return exception.map_remote_error(ex)
 
