@@ -1,3 +1,4 @@
+
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -14,7 +15,6 @@
 from webob import exc
 
 from heat.api.openstack.v1 import util
-from heat.common import serializers
 from heat.common import wsgi
 from heat.rpc import client as rpc_client
 
@@ -85,8 +85,7 @@ class SoftwareDeploymentController(object):
         """
         update_data = dict((k, body.get(k)) for k in (
             'config_id', 'input_values', 'output_values', 'action',
-            'status', 'status_reason')
-            if body.get(k, None) is not None)
+            'status', 'status_reason'))
         sd = self.rpc_client.update_software_deployment(req.context,
                                                         deployment_id,
                                                         **update_data)
@@ -111,6 +110,6 @@ def create_resource(options):
     Software deployments resource factory method.
     """
     deserializer = wsgi.JSONRequestDeserializer()
-    serializer = serializers.JSONResponseSerializer()
+    serializer = wsgi.JSONResponseSerializer()
     return wsgi.Resource(
         SoftwareDeploymentController(options), deserializer, serializer)
