@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,19 +16,18 @@
 Stack endpoint for Heat v1 ReST API.
 """
 
-import itertools
 from webob import exc
 
 from heat.api.openstack.v1 import util
-from heat.common import identifier
-from heat.common import wsgi
-from heat.common import template_format
+from heat.api.openstack.v1.views import stacks_view
 from heat.common import environment_format
+from heat.common import identifier
+from heat.common import template_format
+from heat.common import urlfetch
+from heat.common import wsgi
+from heat.openstack.common import log as logging
 from heat.rpc import api as engine_api
 from heat.rpc import client as rpc_client
-from heat.common import urlfetch
-
-from heat.openstack.common import log as logging
 
 logger = logging.getLogger(__name__)
 
@@ -71,9 +69,10 @@ class InstantiationData(object):
                 return environment_format.parse(data)
             else:
                 return template_format.parse(data)
-        except ValueError:
-            err_reason = _("%s not in valid format") % data_type
-            raise exc.HTTPBadRequest(err_reason)
+        except ValueError as parse_ex:
+            mdict = {'type': data_type, 'error': parse_ex}
+            msg = _("%(type)s not in valid format: %(error)s") % mdict
+            raise exc.HTTPBadRequest(msg)
 
     def stack_name(self):
         """
@@ -136,93 +135,120 @@ class InstantiationData(object):
         return dict((k, v) for k, v in params if k not in self.PARAMS)
 
 
-def format_stack(req, stack, keys=[]):
-    include_key = lambda k: k in keys if keys else True
-
-    def transform(key, value):
-        if not include_key(key):
-            return
-
-        if key == engine_api.STACK_ID:
-            yield ('id', value['stack_id'])
-            yield ('links', [util.make_link(req, value)])
-        elif key == engine_api.STACK_ACTION:
-            return
-        elif (key == engine_api.STACK_STATUS and
-              engine_api.STACK_ACTION in stack):
-            # To avoid breaking API compatibility, we join RES_ACTION
-            # and RES_STATUS, so the API format doesn't expose the
-            # internal split of state into action/status
-            yield (key, '_'.join((stack[engine_api.STACK_ACTION], value)))
-        else:
-            # TODO(zaneb): ensure parameters can be formatted for XML
-            #elif key == engine_api.STACK_PARAMETERS:
-            #    return key, json.dumps(value)
-            yield (key, value)
-
-    return dict(itertools.chain.from_iterable(
-        transform(k, v) for k, v in stack.items()))
-
-
 class StackController(object):
     """
     WSGI controller for stacks resource in Heat v1 API
     Implements the API actions
     """
+    # Define request scope (must match what is in policy.json)
+    REQUEST_SCOPE = 'stacks'
 
     def __init__(self, options):
         self.options = options
-        self.engine = rpc_client.EngineClient()
+        self.rpc_client = rpc_client.EngineClient()
 
     def default(self, req, **args):
         raise exc.HTTPNotFound()
 
-    @util.tenant_local
+    def _index(self, req, tenant_safe=True):
+        filter_whitelist = {
+            'status': 'mixed',
+            'name': 'mixed',
+        }
+        whitelist = {
+            'limit': 'single',
+            'marker': 'single',
+            'sort_dir': 'single',
+            'sort_keys': 'multi',
+        }
+        params = util.get_allowed_params(req.params, whitelist)
+        filter_params = util.get_allowed_params(req.params, filter_whitelist)
+
+        if not filter_params:
+            filter_params = None
+
+        stacks = self.rpc_client.list_stacks(req.context,
+                                             filters=filter_params,
+                                             tenant_safe=tenant_safe,
+                                             **params)
+
+        count = None
+        if req.params.get('with_count'):
+            try:
+                # Check if engine has been updated to a version with
+                # support to count_stacks before trying to use it.
+                count = self.rpc_client.count_stacks(req.context,
+                                                     filters=filter_params,
+                                                     tenant_safe=tenant_safe)
+            except AttributeError as exc:
+                logger.warning("Old Engine Version: %s" % str(exc))
+
+        return stacks_view.collection(req, stacks=stacks, count=count,
+                                      tenant_safe=tenant_safe)
+
+    @util.policy_enforce
+    def global_index(self, req):
+        return self._index(req, tenant_safe=False)
+
+    @util.policy_enforce
     def index(self, req):
         """
         Lists summary information for all stacks
         """
+        global_tenant = bool(req.params.get('global_tenant', False))
+        if global_tenant:
+            return self.global_index(req, req.context.tenant_id)
 
-        stacks = self.engine.list_stacks(req.context)
+        return self._index(req)
 
-        summary_keys = (engine_api.STACK_ID,
-                        engine_api.STACK_NAME,
-                        engine_api.STACK_DESCRIPTION,
-                        engine_api.STACK_STATUS,
-                        engine_api.STACK_STATUS_DATA,
-                        engine_api.STACK_CREATION_TIME,
-                        engine_api.STACK_DELETION_TIME,
-                        engine_api.STACK_UPDATED_TIME)
-
-        return {'stacks': [format_stack(req, s, summary_keys) for s in stacks]}
-
-    @util.tenant_local
+    @util.policy_enforce
     def detail(self, req):
         """
         Lists detailed information for all stacks
         """
-        stacks = self.engine.list_stacks(req.context)
+        stacks = self.rpc_client.list_stacks(req.context)
 
-        return {'stacks': [format_stack(req, s) for s in stacks]}
+        return {'stacks': [stacks_view.format_stack(req, s) for s in stacks]}
 
-    @util.tenant_local
-    def create(self, req, body):
+    @util.policy_enforce
+    def preview(self, req, body):
         """
-        Create a new stack
+        Preview the outcome of a template and its params
         """
 
         data = InstantiationData(body)
 
-        result = self.engine.create_stack(req.context,
-                                          data.stack_name(),
-                                          data.template(),
-                                          data.environment(),
-                                          data.files(),
-                                          data.args())
+        result = self.rpc_client.preview_stack(req.context,
+                                               data.stack_name(),
+                                               data.template(),
+                                               data.environment(),
+                                               data.files(),
+                                               data.args())
 
-        return {'stack': format_stack(req, {engine_api.STACK_ID: result})}
+        formatted_stack = stacks_view.format_stack(req, result)
+        return {'stack': formatted_stack}
 
-    @util.tenant_local
+    @util.policy_enforce
+    def create(self, req, body):
+        """
+        Create a new stack
+        """
+        data = InstantiationData(body)
+
+        result = self.rpc_client.create_stack(req.context,
+                                              data.stack_name(),
+                                              data.template(),
+                                              data.environment(),
+                                              data.files(),
+                                              data.args())
+
+        formatted_stack = stacks_view.format_stack(
+            req,
+            {engine_api.STACK_ID: result}
+        )
+        return {'stack': formatted_stack}
+
+    @util.policy_enforce
     def lookup(self, req, stack_name, path='', body=None):
         """
         Redirect to the canonical URL for a stack
@@ -230,8 +256,8 @@ class StackController(object):
         try:
             identity = dict(identifier.HeatIdentifier.from_arn(stack_name))
         except ValueError:
-            identity = self.engine.identify_stack(req.context,
-                                                  stack_name)
+            identity = self.rpc_client.identify_stack(req.context,
+                                                      stack_name)
 
         location = util.make_url(req, identity)
         if path:
@@ -245,15 +271,15 @@ class StackController(object):
         Gets detailed information for a stack
         """
 
-        stack_list = self.engine.show_stack(req.context,
-                                            identity)
+        stack_list = self.rpc_client.show_stack(req.context,
+                                                identity)
 
         if not stack_list:
             raise exc.HTTPInternalServerError()
 
         stack = stack_list[0]
 
-        return {'stack': format_stack(req, stack)}
+        return {'stack': stacks_view.format_stack(req, stack)}
 
     @util.identified_stack
     def template(self, req, identity):
@@ -261,8 +287,8 @@ class StackController(object):
         Get the template body for an existing stack
         """
 
-        templ = self.engine.get_template(req.context,
-                                         identity)
+        templ = self.rpc_client.get_template(req.context,
+                                             identity)
 
         if templ is None:
             raise exc.HTTPNotFound()
@@ -277,12 +303,12 @@ class StackController(object):
         """
         data = InstantiationData(body)
 
-        res = self.engine.update_stack(req.context,
-                                       identity,
-                                       data.template(),
-                                       data.environment(),
-                                       data.files(),
-                                       data.args())
+        self.rpc_client.update_stack(req.context,
+                                     identity,
+                                     data.template(),
+                                     data.environment(),
+                                     data.files(),
+                                     data.args())
 
         raise exc.HTTPAccepted()
 
@@ -292,16 +318,25 @@ class StackController(object):
         Delete the specified stack
         """
 
-        res = self.engine.delete_stack(req.context,
-                                       identity,
-                                       cast=False)
+        res = self.rpc_client.delete_stack(req.context,
+                                           identity,
+                                           cast=False)
 
         if res is not None:
             raise exc.HTTPBadRequest(res['Error'])
 
         raise exc.HTTPNoContent()
 
-    @util.tenant_local
+    @util.identified_stack
+    def abandon(self, req, identity):
+        """
+        Abandons specified stack by deleting the stack and it's resources
+        from the database, but underlying resources will not be deleted.
+        """
+        return self.rpc_client.abandon_stack(req.context,
+                                             identity)
+
+    @util.policy_enforce
     def validate_template(self, req, body):
         """
         Implements the ValidateTemplate API action
@@ -310,34 +345,38 @@ class StackController(object):
 
         data = InstantiationData(body)
 
-        result = self.engine.validate_template(req.context,
-                                               data.template())
+        result = self.rpc_client.validate_template(req.context,
+                                                   data.template(),
+                                                   data.environment())
 
         if 'Error' in result:
             raise exc.HTTPBadRequest(result['Error'])
 
         return result
 
-    @util.tenant_local
+    @util.policy_enforce
     def list_resource_types(self, req):
         """
         Returns a list of valid resource types that may be used in a template.
         """
-        return {'resource_types': self.engine.list_resource_types(req.context)}
+        support_status = req.params.get('support_status')
+        return {
+            'resource_types':
+            self.rpc_client.list_resource_types(req.context, support_status)}
 
-    @util.tenant_local
+    @util.policy_enforce
     def resource_schema(self, req, type_name):
         """
         Returns the schema of the given resource type.
         """
-        return self.engine.resource_schema(req.context, type_name)
+        return self.rpc_client.resource_schema(req.context, type_name)
 
-    @util.tenant_local
+    @util.policy_enforce
     def generate_template(self, req, type_name):
         """
         Generates a template based on the specified type.
         """
-        return self.engine.generate_template(req.context, type_name)
+        return self.rpc_client.generate_template(req.context, type_name)
 
 
 class StackSerializer(wsgi.JSONResponseSerializer):
@@ -361,7 +400,6 @@ def create_resource(options):
     """
     Stacks resource factory method.
     """
-    # TODO(zaneb) handle XML based on Content-type/Accepts
     deserializer = wsgi.JSONRequestDeserializer()
     serializer = StackSerializer()
     return wsgi.Resource(StackController(options), deserializer, serializer)
