@@ -35,7 +35,7 @@ def init_logging():
     LOG.setLevel(logging.INFO)
     LOG.addHandler(logging.StreamHandler())
     fh = logging.FileHandler("/var/log/heat-provision.log")
-    os.chmod(fh.baseFilename, 0o600)
+    os.chmod(fh.baseFilename, int("600", 8))
     LOG.addHandler(fh)
 
 
@@ -55,14 +55,16 @@ def call(args):
         if data:
             for x in data:
                 ls.write(x)
-    except OSError as ex:
+    except OSError:
+        ex_type, ex, tb = sys.exc_info()
         if ex.errno == errno.ENOEXEC:
             LOG.error('Userdata empty or not executable: %s\n' % str(ex))
             return os.EX_OK
         else:
             LOG.error('OS error running userdata: %s\n' % str(ex))
             return os.EX_OSERR
-    except Exception as ex:
+    except Exception:
+        ex_type, ex, tb = sys.exc_info()
         LOG.error('Unknown error running userdata: %s\n' % str(ex))
         return os.EX_SOFTWARE
     return p.returncode
@@ -77,7 +79,7 @@ def main():
         return -1
 
     userdata_path = os.path.join(VAR_PATH, 'cfn-userdata')
-    os.chmod(userdata_path, 0o700)
+    os.chmod(userdata_path, int("700", 8))
 
     LOG.info('Provision began: %s\n' % datetime.datetime.now())
     returncode = call([userdata_path])
@@ -96,5 +98,8 @@ if __name__ == '__main__':
 
     provision_log = os.path.join(VAR_PATH, 'provision-finished')
     # touch the file so it is timestamped with when finished
-    with file(provision_log, 'a'):
+    pl = file(provision_log, 'a')
+    try:
         os.utime(provision_log, None)
+    finally:
+        pl.close()
