@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -31,8 +30,8 @@ def parse_isoduration(duration):
     """
     result = iso_duration_re.match(duration)
     if not result:
-        raise ValueError('Only ISO 8601 duration format of the form '
-                         'PT#H#M#S is supported.')
+        raise ValueError(_('Only ISO 8601 duration format of the form '
+                           'PT#H#M#S is supported.'))
 
     t = 0
     t += (3600 * int(result.group(1))) if result.group(1) else 0
