@@ -1,4 +1,5 @@
-#
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -19,3 +20,11 @@ def upgrade(migrate_engine):
     software_deployment = sqlalchemy.Table(
         'software_deployment', meta, autoload=True)
     software_deployment.c.signal_id.drop()
+
+
+def downgrade(migrate_engine):
+    meta = sqlalchemy.MetaData(bind=migrate_engine)
+    software_deployment = sqlalchemy.Table(
+        'software_deployment', meta, autoload=True)
+    signal_id = sqlalchemy.Column('signal_id', sqlalchemy.String(1024))
+    signal_id.create(software_deployment)
