@@ -79,6 +79,21 @@ class SoftDelete(object):
                              session=session)
 
 
+class StateAware(object):
+
+    action = sqlalchemy.Column('action', sqlalchemy.String(255))
+    status = sqlalchemy.Column('status', sqlalchemy.String(255))
+    _status_reason = sqlalchemy.Column('status_reason', sqlalchemy.String(255))
+
+    @property
+    def status_reason(self):
+        return self._status_reason
+
+    @status_reason.setter
+    def status_reason(self, reason):
+        self._status_reason = reason and reason[:255] or ''
+
+
 class RawTemplate(BASE, HeatBase):
     """Represents an unparsed template which should be in JSON format."""
 
@@ -88,7 +103,7 @@ class RawTemplate(BASE, HeatBase):
     files = sqlalchemy.Column(Json)
 
 
-class Stack(BASE, HeatBase, SoftDelete):
+class Stack(BASE, HeatBase, SoftDelete, StateAware):
     """Represents a stack created by the heat engine."""
 
     __tablename__ = 'stack'
@@ -103,9 +118,6 @@ class Stack(BASE, HeatBase, SoftDelete):
     raw_template = relationship(RawTemplate, backref=backref('stack'))
     username = sqlalchemy.Column(sqlalchemy.String(256))
     tenant = sqlalchemy.Column(sqlalchemy.String(256))
-    action = sqlalchemy.Column('action', sqlalchemy.String(255))
-    status = sqlalchemy.Column('status', sqlalchemy.String(255))
-    status_reason = sqlalchemy.Column('status_reason', sqlalchemy.String(255))
     parameters = sqlalchemy.Column('parameters', Json)
     user_creds_id = sqlalchemy.Column(
         sqlalchemy.Integer,
@@ -172,10 +184,19 @@ class Event(BASE, HeatBase):
     resource_status = sqlalchemy.Column(sqlalchemy.String(255))
     resource_name = sqlalchemy.Column(sqlalchemy.String(255))
     physical_resource_id = sqlalchemy.Column(sqlalchemy.String(255))
-    resource_status_reason = sqlalchemy.Column(sqlalchemy.String(255))
+    _resource_status_reason = sqlalchemy.Column(
+        'resource_status_reason', sqlalchemy.String(255))
     resource_type = sqlalchemy.Column(sqlalchemy.String(255))
     resource_properties = sqlalchemy.Column(sqlalchemy.PickleType)
 
+    @property
+    def resource_status_reason(self):
+        return self._resource_status_reason
+
+    @resource_status_reason.setter
+    def resource_status_reason(self, reason):
+        self._resource_status_reason = reason and reason[:255] or ''
+
 
 class ResourceData(BASE, HeatBase):
     """Key/value store of arbitrary, resource-specific data."""
@@ -196,7 +217,7 @@ class ResourceData(BASE, HeatBase):
                                     nullable=False)
 
 
-class Resource(BASE, HeatBase):
+class Resource(BASE, HeatBase, StateAware):
     """Represents a resource created by the heat engine."""
 
     __tablename__ = 'resource'
@@ -204,11 +225,8 @@ class Resource(BASE, HeatBase):
     id = sqlalchemy.Column(sqlalchemy.String(36),
                            primary_key=True,
                            default=lambda: str(uuid.uuid4()))
-    action = sqlalchemy.Column('action', sqlalchemy.String(255))
-    status = sqlalchemy.Column('status', sqlalchemy.String(255))
     name = sqlalchemy.Column('name', sqlalchemy.String(255), nullable=True)
     nova_instance = sqlalchemy.Column('nova_instance', sqlalchemy.String(255))
-    status_reason = sqlalchemy.Column('status_reason', sqlalchemy.String(255))
     # odd name as "metadata" is reserved
     rsrc_metadata = sqlalchemy.Column('rsrc_metadata', Json)
 
@@ -274,10 +292,10 @@ class SoftwareConfig(BASE, HeatBase):
     group = sqlalchemy.Column('group', sqlalchemy.String(255))
     config = sqlalchemy.Column('config', Json)
     tenant = sqlalchemy.Column(
-        'tenant', sqlalchemy.String(256), nullable=False)
+        'tenant', sqlalchemy.String(64), nullable=False)
 
 
-class SoftwareDeployment(BASE, HeatBase):
+class SoftwareDeployment(BASE, HeatBase, StateAware):
     """
     Represents applying a software configuration resource to a
     single server resource.
@@ -298,9 +316,6 @@ class SoftwareDeployment(BASE, HeatBase):
     input_values = sqlalchemy.Column('input_values', Json)
     output_values = sqlalchemy.Column('output_values', Json)
     tenant = sqlalchemy.Column(
-        'tenant', sqlalchemy.String(256), nullable=False)
+        'tenant', sqlalchemy.String(64), nullable=False)
     stack_user_project_id = sqlalchemy.Column(sqlalchemy.String(64),
                                               nullable=True)
-    action = sqlalchemy.Column('action', sqlalchemy.String(255))
-    status = sqlalchemy.Column('status', sqlalchemy.String(255))
-    status_reason = sqlalchemy.Column('status_reason', sqlalchemy.String(255))
