@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -24,7 +23,6 @@ gettextutils.install('heat')
 
 from oslo.config import cfg
 from heat.openstack.common import log as logging
-from heat.db import api
 from heat.db import migration
 
 LOG = logging.getLogger(__name__)
@@ -36,8 +34,6 @@ if __name__ == '__main__':
     print('*******************************************', file=sys.stderr)
     cfg.CONF(project='heat', prog='heat-engine')
 
-    api.configure()
-
     try:
         migration.db_sync()
     except Exception as exc:
