@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,13 +15,15 @@
 import collections
 import itertools
 
+from six.moves import xrange
+
 from heat.common import exception
 
 from heat.openstack.common.gettextutils import _
 
 
 class CircularDependencyException(exception.HeatException):
-    message = _("Circular Dependency Found: %(cycle)s")
+    msg_fmt = _("Circular Dependency Found: %(cycle)s")
 
 
 class Node(object):
@@ -128,7 +129,9 @@ class Graph(collections.defaultdict):
         node = self[key]
 
         for src in node.required_by():
-            self[src] -= key
+            src_node = self[src]
+            if key in src_node:
+                src_node -= key
 
         return super(Graph, self).__delitem__(key)
 
