@@ -11,31 +11,24 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_config import cfg
+from oslo.config import cfg
 
-from heat.common.i18n import _
-from heat.common import messaging
+from heat.openstack.common import log
+from heat.openstack.common.notifier import api as notifier_api
 
+LOG = log.getLogger(__name__)
 SERVICE = 'orchestration'
-INFO = 'INFO'
-ERROR = 'ERROR'
-
-notifier_opts = [
-    cfg.StrOpt('default_notification_level',
-               default=INFO,
-               help=_('Default notification level for outgoing'
-                      'notifications.')),
-    cfg.StrOpt('default_publisher_id',
-               help=_('Default publisher_id for outgoing notifications.')),
-]
 CONF = cfg.CONF
-CONF.register_opts(notifier_opts)
+CONF.import_opt('default_notification_level',
+                'heat.openstack.common.notifier.api')
+CONF.import_opt('default_publisher_id',
+                'heat.openstack.common.notifier.api')
 
 
 def _get_default_publisher():
     publisher_id = CONF.default_publisher_id
     if publisher_id is None:
-        publisher_id = "%s.%s" % (SERVICE, CONF.host)
+        publisher_id = notifier_api.publisher_id(SERVICE)
     return publisher_id
 
 
@@ -44,11 +37,7 @@ def get_default_level():
 
 
 def notify(context, event_type, level, body):
-    client = messaging.get_notifier(_get_default_publisher())
 
-    method = getattr(client, level.lower())
-    method(context, "%s.%s" % (SERVICE, event_type), body)
-
-
-def list_opts():
-    yield None, notifier_opts
+    notifier_api.notify(context, _get_default_publisher(),
+                        "%s.%s" % (SERVICE, event_type),
+                        level, body)
