@@ -11,6 +11,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from heat.openstack.common.notifier import api as notifier_api
+
 from heat.engine import api as engine_api
 from heat.engine import notification
 
@@ -29,7 +31,7 @@ def send(stack):
         suffix = 'end'
     else:
         suffix = 'error'
-        level = notification.ERROR
+        level = notifier_api.ERROR
 
     event_type = '%s.%s.%s' % ('stack',
                                stack.action.lower(),
