@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -53,7 +52,7 @@ class Timeout(BaseException):
 
     This allows the task to perform any necessary cleanup, as well as use a
     different exception to notify the controlling task if appropriate. If the
-    task supresses the exception altogether, it will be cancelled but the
+    task suppresses the exception altogether, it will be cancelled but the
     controlling task will not be notified of the timeout.
     """
 
@@ -75,6 +74,29 @@ class Timeout(BaseException):
         return wallclock() > self._endtime
 
 
+class ExceptionGroup(Exception):
+    '''
+    Container for multiple exceptions.
+
+    This exception is used by DependencyTaskGroup when the flag
+    aggregate_exceptions is set to True and it's re-raised again when all tasks
+    are finished.  This way it can be caught later on so that the individual
+    exceptions can be acted upon.
+    '''
+
+    def __init__(self, exceptions=None):
+        if exceptions is None:
+            exceptions = list()
+
+        self.exceptions = list(exceptions)
+
+    def __str__(self):
+        return str(map(str, self.exceptions))
+
+    def __unicode__(self):
+        return unicode(map(str, self.exceptions))
+
+
 class TaskRunner(object):
     """
     Wrapper for a resumable task (co-routine).
@@ -105,7 +127,7 @@ class TaskRunner(object):
     def _sleep(self, wait_time):
         """Sleep for the specified number of seconds."""
         if ENABLE_SLEEP and wait_time is not None:
-            logger.debug('%s sleeping' % str(self))
+            logger.debug(_('%s sleeping') % str(self))
             eventlet.sleep(wait_time)
 
     def __call__(self, wait_time=1, timeout=None):
@@ -128,7 +150,7 @@ class TaskRunner(object):
         """
         assert self._runner is None, "Task already started"
 
-        logger.debug('%s starting' % str(self))
+        logger.debug(_('%s starting') % str(self))
 
         if timeout is not None:
             self._timeout = Timeout(self, timeout)
@@ -140,7 +162,7 @@ class TaskRunner(object):
         else:
             self._runner = False
             self._done = True
-            logger.debug('%s done (not resumable)' % str(self))
+            logger.debug(_('%s done (not resumable)') % str(self))
 
     def step(self):
         """
@@ -151,7 +173,7 @@ class TaskRunner(object):
             assert self._runner is not None, "Task not started"
 
             if self._timeout is not None and self._timeout.expired():
-                logger.info('%s timed out' % str(self))
+                logger.info(_('%s timed out') % str(self))
 
                 try:
                     self._runner.throw(self._timeout)
@@ -161,13 +183,13 @@ class TaskRunner(object):
                     # Clean up in case task swallows exception without exiting
                     self.cancel()
             else:
-                logger.debug('%s running' % str(self))
+                logger.debug(_('%s running') % str(self))
 
                 try:
                     next(self._runner)
                 except StopIteration:
                     self._done = True
-                    logger.debug('%s complete' % str(self))
+                    logger.debug(_('%s complete') % str(self))
 
         return self._done
 
@@ -182,11 +204,14 @@ class TaskRunner(object):
             self._sleep(wait_time)
 
     def cancel(self):
-        """Cancel the task if it is running."""
-        if self.started() and not self.done():
-            logger.debug('%s cancelled' % str(self))
-            self._runner.close()
-            self._done = True
+        """Cancel the task and mark it as done."""
+        if not self.done():
+            logger.debug(_('%s cancelled') % str(self))
+            try:
+                if self.started():
+                    self._runner.close()
+            finally:
+                self._done = True
 
     def started(self):
         """Return True if the task has been started."""
@@ -207,7 +232,7 @@ def wrappertask(task):
 
     This is essentially a replacement for the Python 3-only "yield from"
     keyword (PEP 380), using the "yield" keyword that is supported in
-    Python 2. For example:
+    Python 2. For example::
 
         @wrappertask
         def parent_task(self):
@@ -236,9 +261,9 @@ def wrappertask(task):
                     while subtask_running:
                         try:
                             yield step
-                        except GeneratorExit as exit:
+                        except GeneratorExit as ex:
                             subtask.close()
-                            raise exit
+                            raise ex
                         except:
                             try:
                                 step = subtask.throw(*sys.exc_info())
@@ -251,9 +276,9 @@ def wrappertask(task):
                                 subtask_running = False
                 else:
                     yield
-            except GeneratorExit as exit:
+            except GeneratorExit as ex:
                 parent.close()
-                raise exit
+                raise ex
             except:
                 subtask = parent.throw(*sys.exc_info())
             else:
@@ -268,7 +293,7 @@ class DependencyTaskGroup(object):
     """
 
     def __init__(self, dependencies, task=lambda o: o(),
-                 reverse=False, name=None):
+                 reverse=False, name=None, aggregate_exceptions=False):
         """
         Initialise with the task dependencies and (optionally) a task to run on
         each.
@@ -276,9 +301,14 @@ class DependencyTaskGroup(object):
         If no task is supplied, it is assumed that the tasks are stored
         directly in the dependency tree. If a task is supplied, the object
         stored in the dependency tree is passed as an argument.
+
+        If aggregate_exceptions is set to True, then all the tasks will be run
+        and any raised exceptions will be stored to be re-raised after all
+        tasks are done.
         """
         self._runners = dict((o, TaskRunner(task, o)) for o in dependencies)
         self._graph = dependencies.graph(reverse=reverse)
+        self.aggregate_exceptions = aggregate_exceptions
 
         if name is None:
             name = '(%s) %s' % (getattr(task, '__name__',
@@ -292,21 +322,40 @@ class DependencyTaskGroup(object):
 
     def __call__(self):
         """Return a co-routine which runs the task group."""
+        raised_exceptions = []
         try:
             while any(self._runners.itervalues()):
-                for k, r in self._ready():
-                    r.start()
+                try:
+                    for k, r in self._ready():
+                        r.start()
 
-                yield
+                    yield
 
-                for k, r in self._running():
-                    if r.step():
-                        del self._graph[k]
+                    for k, r in self._running():
+                        if r.step():
+                            del self._graph[k]
+                except Exception as e:
+                    self._cancel_recursively(k, r)
+                    if not self.aggregate_exceptions:
+                        raise
+                    raised_exceptions.append(e)
         except:
             with excutils.save_and_reraise_exception():
                 for r in self._runners.itervalues():
                     r.cancel()
 
+        if raised_exceptions:
+            raise ExceptionGroup(raised_exceptions)
+
+    def _cancel_recursively(self, key, runner):
+        runner.cancel()
+        node = self._graph[key]
+        for dependent_node in node.required_by():
+            node_runner = self._runners[dependent_node]
+            self._cancel_recursively(dependent_node, node_runner)
+
+        del self._graph[key]
+
     def _ready(self):
         """
         Iterate over all subtasks that are ready to start - i.e. all their
