@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 # Copyright 2012, Red Hat, Inc.
 #
@@ -26,9 +25,10 @@ import heat.openstack.common.rpc.proxy
 class EngineClient(heat.openstack.common.rpc.proxy.RpcProxy):
     '''Client side of the heat engine rpc API.
 
-    API version history:
+    API version history::
 
         1.0 - Initial version.
+        1.1 - Add support_status argument to list_resource_types()
     '''
 
     BASE_RPC_API_VERSION = '1.0'
@@ -50,24 +50,68 @@ class EngineClient(heat.openstack.common.rpc.proxy.RpcProxy):
         return self.call(ctxt, self.make_msg('identify_stack',
                                              stack_name=stack_name))
 
-    def list_stacks(self, ctxt):
+    def list_stacks(self, ctxt, limit=None, marker=None, sort_keys=None,
+                    sort_dir=None, filters=None, tenant_safe=True):
         """
-        The list_stacks method returns the attributes of all stacks.
+        The list_stacks method returns attributes of all stacks.  It supports
+        pagination (``limit`` and ``marker``), sorting (``sort_keys`` and
+        ``sort_dir``) and filtering (``filters``) of the results.
 
         :param ctxt: RPC context.
+        :param limit: the number of stacks to list (integer or string)
+        :param marker: the ID of the last item in the previous page
+        :param sort_keys: an array of fields used to sort the list
+        :param sort_dir: the direction of the sort ('asc' or 'desc')
+        :param filters: a dict with attribute:value to filter the list
+        :param tenant_safe: if true, scope the request by the current tenant
+        :returns: a list of stacks
         """
-        return self.call(ctxt, self.make_msg('list_stacks'))
+        return self.call(ctxt, self.make_msg('list_stacks', limit=limit,
+                         sort_keys=sort_keys, marker=marker,
+                         sort_dir=sort_dir, filters=filters,
+                         tenant_safe=tenant_safe))
+
+    def count_stacks(self, ctxt, filters=None, tenant_safe=True):
+        """
+        Return the number of stacks that match the given filters
+        :param ctxt: RPC context.
+        :param filters: a dict of ATTR:VALUE to match against stacks
+        :param tenant_safe: if true, scope the request by the current tenant
+        :returns: a integer representing the number of matched stacks
+        """
+        return self.call(ctxt, self.make_msg('count_stacks',
+                                             filters=filters,
+                                             tenant_safe=tenant_safe))
 
     def show_stack(self, ctxt, stack_identity):
         """
         Return detailed information about one or all stacks.
         :param ctxt: RPC context.
         :param stack_identity: Name of the stack you want to show, or None to
-                               show all
+        show all
         """
         return self.call(ctxt, self.make_msg('show_stack',
                                              stack_identity=stack_identity))
 
+    def preview_stack(self, ctxt, stack_name, template, params, files, args):
+        """
+        Simulates a new stack using the provided template.
+
+        Note that at this stage the template has already been fetched from the
+        heat-api process if using a template-url.
+
+        :param ctxt: RPC context.
+        :param stack_name: Name of the stack you want to create.
+        :param template: Template of stack you want to create.
+        :param params: Stack Input Params/Environment
+        :param files: files referenced from the environment.
+        :param args: Request parameters/args passed from API
+        """
+        return self.call(ctxt,
+                         self.make_msg('preview_stack', stack_name=stack_name,
+                                       template=template,
+                                       params=params, files=files, args=args))
+
     def create_stack(self, ctxt, stack_name, template, params, files, args):
         """
         The create_stack method creates a new stack using the template
@@ -109,16 +153,18 @@ class EngineClient(heat.openstack.common.rpc.proxy.RpcProxy):
                                              files=files,
                                              args=args))
 
-    def validate_template(self, ctxt, template):
+    def validate_template(self, ctxt, template, params=None):
         """
         The validate_template method uses the stack parser to check
         the validity of a template.
 
         :param ctxt: RPC context.
         :param template: Template of stack you want to create.
+        :param params: Stack Input Params/Environment
         """
         return self.call(ctxt, self.make_msg('validate_template',
-                                             template=template))
+                                             template=template,
+                                             params=params))
 
     def authenticated_to_backend(self, ctxt):
         """
@@ -152,13 +198,27 @@ class EngineClient(heat.openstack.common.rpc.proxy.RpcProxy):
                           self.make_msg('delete_stack',
                                         stack_identity=stack_identity))
 
-    def list_resource_types(self, ctxt):
+    def abandon_stack(self, ctxt, stack_identity):
+        """
+        The abandon_stack method deletes a given stack but
+        resources would not be deleted.
+
+        :param ctxt: RPC context.
+        :param stack_identity: Name of the stack you want to abandon.
+        """
+        return self.call(ctxt,
+                         self.make_msg('abandon_stack',
+                                       stack_identity=stack_identity))
+
+    def list_resource_types(self, ctxt, support_status=None):
         """
         Get a list of valid resource types.
 
         :param ctxt: RPC context.
         """
-        return self.call(ctxt, self.make_msg('list_resource_types'))
+        return self.call(ctxt, self.make_msg('list_resource_types',
+                                             support_status=support_status),
+                         version='1.1')
 
     def resource_schema(self, ctxt, type_name):
         """
@@ -311,3 +371,67 @@ class EngineClient(heat.openstack.common.rpc.proxy.RpcProxy):
         return self.call(ctxt, self.make_msg('set_watch_state',
                                              watch_name=watch_name,
                                              state=state))
+
+    def get_revision(self, ctxt):
+        return self.call(ctxt, self.make_msg('get_revision'))
+
+    def show_software_config(self, cnxt, config_id):
+        return self.call(cnxt, self.make_msg('show_software_config',
+                                             config_id=config_id))
+
+    def create_software_config(self, cnxt, group, name, config,
+                               inputs=[], outputs=[], options={}):
+        return self.call(cnxt, self.make_msg('create_software_config',
+                                             group=group,
+                                             name=name,
+                                             config=config,
+                                             inputs=inputs,
+                                             outputs=outputs,
+                                             options=options))
+
+    def delete_software_config(self, cnxt, config_id):
+        return self.call(cnxt, self.make_msg('delete_software_config',
+                                             config_id=config_id))
+
+    def list_software_deployments(self, cnxt, server_id=None):
+        return self.call(cnxt, self.make_msg('list_software_deployments',
+                                             server_id=server_id))
+
+    def metadata_software_deployments(self, cnxt, server_id):
+        return self.call(cnxt, self.make_msg('metadata_software_deployments',
+                                             server_id=server_id))
+
+    def show_software_deployment(self, cnxt, deployment_id):
+        return self.call(cnxt, self.make_msg('show_software_deployment',
+                                             deployment_id=deployment_id))
+
+    def create_software_deployment(self, cnxt, server_id, config_id=None,
+                                   input_values={}, action='INIT',
+                                   status='COMPLETE', status_reason='',
+                                   stack_user_project_id=None):
+        return self.call(cnxt, self.make_msg(
+            'create_software_deployment',
+            server_id=server_id,
+            config_id=config_id,
+            input_values=input_values,
+            action=action,
+            status=status,
+            status_reason=status_reason,
+            stack_user_project_id=stack_user_project_id))
+
+    def update_software_deployment(self, cnxt, deployment_id,
+                                   config_id=None, input_values=None,
+                                   output_values=None, action=None,
+                                   status=None, status_reason=None):
+        return self.call(cnxt, self.make_msg('update_software_deployment',
+                                             deployment_id=deployment_id,
+                                             config_id=config_id,
+                                             input_values=input_values,
+                                             output_values=output_values,
+                                             action=action,
+                                             status=status,
+                                             status_reason=status_reason))
+
+    def delete_software_deployment(self, cnxt, deployment_id):
+        return self.call(cnxt, self.make_msg('delete_software_deployment',
+                                             deployment_id=deployment_id))
