@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -13,28 +12,36 @@
 #    under the License.
 
 
-import fixtures
 import logging
-import mox
 import os
 import sys
+import time
+
+import fixtures
+import mox
+from oslo.config import cfg
+import testscenarios
 import testtools
 
-from oslo.config import cfg
-
-import heat.engine.scheduler as scheduler
 from heat.engine import environment
 from heat.engine import resources
+from heat.engine import scheduler
+from heat.openstack.common.fixture import mockpatch
 
 
-class HeatTestCase(testtools.TestCase):
+class HeatTestCase(testscenarios.WithScenarios, testtools.TestCase):
+
+    TIME_STEP = 0.1
 
     def setUp(self):
         super(HeatTestCase, self).setUp()
         self.m = mox.Mox()
         self.addCleanup(self.m.UnsetStubs)
-        self.useFixture(fixtures.FakeLogger(level=logging.DEBUG))
+        self.logger = self.useFixture(fixtures.FakeLogger(level=logging.DEBUG))
         scheduler.ENABLE_SLEEP = False
+        self.useFixture(fixtures.MonkeyPatch(
+            'heat.common.exception._FATAL_EXCEPTION_FORMAT_ERRORS',
+            True))
 
         def enable_sleep():
             scheduler.ENABLE_SLEEP = True
@@ -47,6 +54,8 @@ class HeatTestCase(testtools.TestCase):
                                'environment.d')
 
         cfg.CONF.set_default('environment_dir', env_dir)
+        cfg.CONF.set_override('allowed_rpc_exception_modules',
+                              ['heat.common.exception', 'exceptions'])
         self.addCleanup(cfg.CONF.reset)
 
         tri = resources.global_env().get_resource_info(
@@ -58,3 +67,20 @@ class HeatTestCase(testtools.TestCase):
             if templ_path not in cur_path:
                 tri.template_name = cur_path.replace('/etc/heat/templates',
                                                      templ_path)
+
+    def stub_wallclock(self):
+        """
+        Overrides scheduler wallclock to speed up tests expecting timeouts.
+        """
+        self._wallclock = time.time()
+
+        def fake_wallclock():
+            self._wallclock += self.TIME_STEP
+            return self._wallclock
+
+        self.m.StubOutWithMock(scheduler, 'wallclock')
+        scheduler.wallclock = fake_wallclock
+
+    def patchobject(self, obj, attr):
+        mockfixture = self.useFixture(mockpatch.PatchObject(obj, attr))
+        return mockfixture.mock
