@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -13,17 +12,16 @@
 #    under the License.
 
 
-from heat.tests.common import HeatTestCase
-
-import requests
 import json
-from oslo.config import cfg
 
+from oslo.config import cfg
+import requests
+
+from heat.api.aws import ec2token
 from heat.api.aws import exception
 from heat.common.wsgi import Request
-from heat.api.aws import ec2token
-
 from heat.openstack.common import importutils
+from heat.tests.common import HeatTestCase
 
 
 class Ec2TokenTest(HeatTestCase):
@@ -45,40 +43,40 @@ class Ec2TokenTest(HeatTestCase):
     def test_conf_get_paste(self):
         dummy_conf = {'auth_uri': 'http://192.0.2.9/v2.0'}
         ec2 = ec2token.EC2Token(app=None, conf=dummy_conf)
-        self.assertEqual(ec2._conf_get('auth_uri'), 'http://192.0.2.9/v2.0')
+        self.assertEqual('http://192.0.2.9/v2.0', ec2._conf_get('auth_uri'))
         self.assertEqual(
-            ec2._conf_get_keystone_ec2_uri('http://192.0.2.9/v2.0'),
-            'http://192.0.2.9/v2.0/ec2tokens')
+            'http://192.0.2.9/v2.0/ec2tokens',
+            ec2._conf_get_keystone_ec2_uri('http://192.0.2.9/v2.0'))
 
     def test_conf_get_opts(self):
         cfg.CONF.set_default('auth_uri', 'http://192.0.2.9/v2.0/',
                              group='ec2authtoken')
         ec2 = ec2token.EC2Token(app=None, conf={})
-        self.assertEqual(ec2._conf_get('auth_uri'), 'http://192.0.2.9/v2.0/')
+        self.assertEqual('http://192.0.2.9/v2.0/', ec2._conf_get('auth_uri'))
         self.assertEqual(
-            ec2._conf_get_keystone_ec2_uri('http://192.0.2.9/v2.0/'),
-            'http://192.0.2.9/v2.0/ec2tokens')
+            'http://192.0.2.9/v2.0/ec2tokens',
+            ec2._conf_get_keystone_ec2_uri('http://192.0.2.9/v2.0/'))
 
     def test_get_signature_param_old(self):
         params = {'Signature': 'foo'}
         dummy_req = self._dummy_GET_request(params)
         ec2 = ec2token.EC2Token(app=None, conf={})
-        self.assertEqual(ec2._get_signature(dummy_req), 'foo')
+        self.assertEqual('foo', ec2._get_signature(dummy_req))
 
     def test_get_signature_param_new(self):
         params = {'X-Amz-Signature': 'foo'}
         dummy_req = self._dummy_GET_request(params)
         ec2 = ec2token.EC2Token(app=None, conf={})
-        self.assertEqual(ec2._get_signature(dummy_req), 'foo')
+        self.assertEqual('foo', ec2._get_signature(dummy_req))
 
     def test_get_signature_header_space(self):
         req_env = {'HTTP_AUTHORIZATION':
                    ('Authorization: foo  Credential=foo/bar, '
-                   'SignedHeaders=content-type;host;x-amz-date, '
-                   'Signature=xyz')}
+                    'SignedHeaders=content-type;host;x-amz-date, '
+                    'Signature=xyz')}
         dummy_req = self._dummy_GET_request(environ=req_env)
         ec2 = ec2token.EC2Token(app=None, conf={})
-        self.assertEqual(ec2._get_signature(dummy_req), 'xyz')
+        self.assertEqual('xyz', ec2._get_signature(dummy_req))
 
     def test_get_signature_header_notlast(self):
         req_env = {'HTTP_AUTHORIZATION':
@@ -87,66 +85,66 @@ class Ec2TokenTest(HeatTestCase):
                     'SignedHeaders=content-type;host;x-amz-date ')}
         dummy_req = self._dummy_GET_request(environ=req_env)
         ec2 = ec2token.EC2Token(app=None, conf={})
-        self.assertEqual(ec2._get_signature(dummy_req), 'xyz')
+        self.assertEqual('xyz', ec2._get_signature(dummy_req))
 
     def test_get_signature_header_nospace(self):
         req_env = {'HTTP_AUTHORIZATION':
                    ('Authorization: foo  Credential=foo/bar,'
-                   'SignedHeaders=content-type;host;x-amz-date,'
-                   'Signature=xyz')}
+                    'SignedHeaders=content-type;host;x-amz-date,'
+                    'Signature=xyz')}
         dummy_req = self._dummy_GET_request(environ=req_env)
         ec2 = ec2token.EC2Token(app=None, conf={})
-        self.assertEqual(ec2._get_signature(dummy_req), 'xyz')
+        self.assertEqual('xyz', ec2._get_signature(dummy_req))
 
     def test_get_access_param_old(self):
         params = {'AWSAccessKeyId': 'foo'}
         dummy_req = self._dummy_GET_request(params)
         ec2 = ec2token.EC2Token(app=None, conf={})
-        self.assertEqual(ec2._get_access(dummy_req), 'foo')
+        self.assertEqual('foo', ec2._get_access(dummy_req))
 
     def test_get_access_param_new(self):
         params = {'X-Amz-Credential': 'foo/bar'}
         dummy_req = self._dummy_GET_request(params)
         ec2 = ec2token.EC2Token(app=None, conf={})
-        self.assertEqual(ec2._get_access(dummy_req), 'foo')
+        self.assertEqual('foo', ec2._get_access(dummy_req))
 
     def test_get_access_header_space(self):
         req_env = {'HTTP_AUTHORIZATION':
                    ('Authorization: foo  Credential=foo/bar, '
-                   'SignedHeaders=content-type;host;x-amz-date, '
-                   'Signature=xyz')}
+                    'SignedHeaders=content-type;host;x-amz-date, '
+                    'Signature=xyz')}
         dummy_req = self._dummy_GET_request(environ=req_env)
         ec2 = ec2token.EC2Token(app=None, conf={})
-        self.assertEqual(ec2._get_access(dummy_req), 'foo')
+        self.assertEqual('foo', ec2._get_access(dummy_req))
 
     def test_get_access_header_nospace(self):
         req_env = {'HTTP_AUTHORIZATION':
                    ('Authorization: foo  Credential=foo/bar,'
-                   'SignedHeaders=content-type;host;x-amz-date,'
-                   'Signature=xyz')}
+                    'SignedHeaders=content-type;host;x-amz-date,'
+                    'Signature=xyz')}
         dummy_req = self._dummy_GET_request(environ=req_env)
         ec2 = ec2token.EC2Token(app=None, conf={})
-        self.assertEqual(ec2._get_access(dummy_req), 'foo')
+        self.assertEqual('foo', ec2._get_access(dummy_req))
 
     def test_get_access_header_last(self):
         req_env = {'HTTP_AUTHORIZATION':
                    ('Authorization: foo '
-                   'SignedHeaders=content-type;host;x-amz-date,'
-                   'Signature=xyz,Credential=foo/bar')}
+                    'SignedHeaders=content-type;host;x-amz-date,'
+                    'Signature=xyz,Credential=foo/bar')}
         dummy_req = self._dummy_GET_request(environ=req_env)
         ec2 = ec2token.EC2Token(app=None, conf={})
-        self.assertEqual(ec2._get_access(dummy_req), 'foo')
+        self.assertEqual('foo', ec2._get_access(dummy_req))
 
     def test_call_x_auth_user(self):
         req_env = {'HTTP_X_AUTH_USER': 'foo'}
         dummy_req = self._dummy_GET_request(environ=req_env)
         ec2 = ec2token.EC2Token(app='xyz', conf={})
-        self.assertEqual(ec2.__call__(dummy_req), 'xyz')
+        self.assertEqual('xyz', ec2.__call__(dummy_req))
 
     def test_call_auth_nosig(self):
         req_env = {'HTTP_AUTHORIZATION':
                    ('Authorization: foo  Credential=foo/bar, '
-                   'SignedHeaders=content-type;host;x-amz-date')}
+                    'SignedHeaders=content-type;host;x-amz-date')}
         dummy_req = self._dummy_GET_request(environ=req_env)
         ec2 = ec2token.EC2Token(app='xyz', conf={})
         self.assertRaises(exception.HeatIncompleteSignatureError,
@@ -155,8 +153,8 @@ class Ec2TokenTest(HeatTestCase):
     def test_call_auth_nouser(self):
         req_env = {'HTTP_AUTHORIZATION':
                    ('Authorization: foo '
-                   'SignedHeaders=content-type;host;x-amz-date,'
-                   'Signature=xyz')}
+                    'SignedHeaders=content-type;host;x-amz-date,'
+                    'Signature=xyz')}
         dummy_req = self._dummy_GET_request(environ=req_env)
         ec2 = ec2token.EC2Token(app='xyz', conf={})
         self.assertRaises(exception.HeatMissingAuthenticationTokenError,
@@ -175,11 +173,11 @@ class Ec2TokenTest(HeatTestCase):
         req_env = {'HTTP_X_AUTH_USER': 'foo',
                    'HTTP_AUTHORIZATION':
                    ('Authorization: foo '
-                   'SignedHeaders=content-type;host;x-amz-date,'
-                   'Signature=xyz')}
+                    'SignedHeaders=content-type;host;x-amz-date,'
+                    'Signature=xyz')}
         dummy_req = self._dummy_GET_request(environ=req_env)
         ec2 = ec2token.EC2Token(app='xyz', conf={})
-        self.assertEqual(ec2.__call__(dummy_req), 'xyz')
+        self.assertEqual('xyz', ec2.__call__(dummy_req))
 
     def _stub_http_connection(self, headers={}, params={}, response=None,
                               req_url='http://123:5000/v2.0/ec2tokens'):
@@ -224,7 +222,7 @@ class Ec2TokenTest(HeatTestCase):
         self._stub_http_connection(headers={'Authorization': auth_str},
                                    response=ok_resp)
         self.m.ReplayAll()
-        self.assertEqual(ec2.__call__(dummy_req), 'woot')
+        self.assertEqual('woot', ec2.__call__(dummy_req))
 
         self.assertEqual('tenant', dummy_req.headers['X-Tenant-Name'])
         self.assertEqual('abcd1234', dummy_req.headers['X-Tenant-Id'])
@@ -252,7 +250,7 @@ class Ec2TokenTest(HeatTestCase):
         self._stub_http_connection(headers={'Authorization': auth_str},
                                    response=ok_resp)
         self.m.ReplayAll()
-        self.assertEqual(ec2.__call__(dummy_req), 'woot')
+        self.assertEqual('woot', ec2.__call__(dummy_req))
 
         self.assertEqual('aa,bb,cc', dummy_req.headers['X-Roles'])
         self.m.VerifyAll()
@@ -340,7 +338,7 @@ class Ec2TokenTest(HeatTestCase):
         self._stub_http_connection(response=ok_resp,
                                    params={'AWSAccessKeyId': 'foo'})
         self.m.ReplayAll()
-        self.assertEqual(ec2.__call__(dummy_req), 'woot')
+        self.assertEqual('woot', ec2.__call__(dummy_req))
 
         self.m.VerifyAll()
 
@@ -376,7 +374,7 @@ class Ec2TokenTest(HeatTestCase):
             params={'AWSAccessKeyId': 'foo'})
 
         self.m.ReplayAll()
-        self.assertEqual(ec2.__call__(dummy_req), 'woot')
+        self.assertEqual('woot', ec2.__call__(dummy_req))
 
         self.m.VerifyAll()
 
@@ -446,7 +444,7 @@ class Ec2TokenTest(HeatTestCase):
         self.m.ReplayAll()
         ex = self.assertRaises(exception.HeatInternalFailureError,
                                ec2.__call__, dummy_req)
-        self.assertEqual(str(ex), 'Service misconfigured')
+        self.assertEqual('Service misconfigured', str(ex))
 
         self.m.VerifyAll()
 
@@ -467,7 +465,7 @@ class Ec2TokenTest(HeatTestCase):
         self._stub_http_connection(response=ok_resp,
                                    params={'AWSAccessKeyId': 'foo'})
         self.m.ReplayAll()
-        self.assertEqual(ec2.__call__(dummy_req), 'woot')
+        self.assertEqual('woot', ec2.__call__(dummy_req))
 
         self.m.VerifyAll()
 
@@ -490,6 +488,6 @@ class Ec2TokenTest(HeatTestCase):
         self._stub_http_connection(response=ok_resp,
                                    params={'AWSAccessKeyId': 'foo'})
         self.m.ReplayAll()
-        self.assertEqual(ec2.__call__(dummy_req), 'woot')
+        self.assertEqual('woot', ec2.__call__(dummy_req))
 
         self.m.VerifyAll()
