@@ -1,4 +1,3 @@
-#
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,12 +14,10 @@
 # limitations under the License.
 
 import mock
-import six
 import webob
-from webob import exc
 
 from heat.common import auth_url
-from heat.tests import common
+from heat.tests.common import HeatTestCase
 
 
 class FakeApp(object):
@@ -29,11 +26,11 @@ class FakeApp(object):
     def __call__(self, environ, start_response):
         """Assert that headers are correctly set up when finally called."""
         resp = webob.Response()
-        resp.body = six.b('SUCCESS')
+        resp.body = 'SUCCESS'
         return resp(environ, start_response)
 
 
-class AuthUrlFilterTest(common.HeatTestCase):
+class AuthUrlFilterTest(HeatTestCase):
 
     def setUp(self):
         super(AuthUrlFilterTest, self).setUp()
@@ -41,27 +38,9 @@ class AuthUrlFilterTest(common.HeatTestCase):
         self.config = {'auth_uri': 'foobar'}
         self.middleware = auth_url.AuthUrlFilter(self.app, self.config)
 
-    @mock.patch.object(auth_url.cfg, 'CONF')
-    def test_adds_default_auth_url_from_clients_keystone(self, mock_cfg):
-        self.config = {}
-        mock_cfg.clients_keystone.auth_uri = 'foobar'
-        mock_cfg.keystone_authtoken.auth_uri = 'this-should-be-ignored'
-        mock_cfg.auth_password.multi_cloud = False
-        with mock.patch('keystoneauth1.discover.Discover') as discover:
-            class MockDiscover(object):
-                def url_for(self, endpoint):
-                    return 'foobar/v3'
-            discover.return_value = MockDiscover()
-            self.middleware = auth_url.AuthUrlFilter(self.app, self.config)
-            req = webob.Request.blank('/tenant_id/')
-            self.middleware(req)
-            self.assertIn('X-Auth-Url', req.headers)
-            self.assertEqual('foobar/v3', req.headers['X-Auth-Url'])
-
     @mock.patch.object(auth_url.cfg, 'CONF')
     def test_adds_default_auth_url_from_keystone_authtoken(self, mock_cfg):
         self.config = {}
-        mock_cfg.clients_keystone.auth_uri = ''
         mock_cfg.keystone_authtoken.auth_uri = 'foobar'
         mock_cfg.auth_password.multi_cloud = False
         self.middleware = auth_url.AuthUrlFilter(self.app, self.config)
@@ -103,11 +82,11 @@ class AuthUrlFilterTest(common.HeatTestCase):
         self.assertTrue(mock_validate.called)
 
     def test_validate_auth_url_with_missing_url(self):
-        self.assertRaises(exc.HTTPBadRequest,
+        self.assertRaises(auth_url.HTTPBadRequest,
                           self.middleware._validate_auth_url,
                           auth_url='')
 
-        self.assertRaises(exc.HTTPBadRequest,
+        self.assertRaises(auth_url.HTTPBadRequest,
                           self.middleware._validate_auth_url,
                           auth_url=None)
 
@@ -115,7 +94,7 @@ class AuthUrlFilterTest(common.HeatTestCase):
     def test_validate_auth_url_with_url_not_allowed(self, mock_cfg):
         mock_cfg.auth_password.allowed_auth_uris = ['foobar']
 
-        self.assertRaises(exc.HTTPUnauthorized,
+        self.assertRaises(auth_url.HTTPUnauthorized,
                           self.middleware._validate_auth_url,
                           auth_url='not foobar')
 
