@@ -17,8 +17,8 @@ import os.path
 
 from oslo.config import cfg
 
-from heat.common import policy
 from heat.common import exception
+from heat.common import policy
 from heat.openstack.common import policy as base_policy
 from heat.tests.common import HeatTestCase
 from heat.tests import utils
@@ -46,25 +46,26 @@ class TestPolicyEnforcer(HeatTestCase):
             cfg.StrOpt('project', default='heat'),
         ]
         cfg.CONF.register_opts(opts)
+        self.addCleanup(self.m.VerifyAll)
 
-    def test_policy_cfn_default(self):
-        pf = policy_path + 'deny_stack_user.json'
+    def stub_policyfile(self, filename):
+        pf = policy_path + filename
         self.m.StubOutWithMock(base_policy.Enforcer, '_get_policy_path')
         base_policy.Enforcer._get_policy_path().MultipleTimes().AndReturn(pf)
         self.m.ReplayAll()
 
+    def test_policy_cfn_default(self):
+        self.stub_policyfile('deny_stack_user.json')
+
         enforcer = policy.Enforcer(scope='cloudformation')
 
         ctx = utils.dummy_context(roles=[])
         for action in self.cfn_actions:
             # Everything should be allowed
-            enforcer.enforce(ctx, action, {})
+            enforcer.enforce(ctx, action)
 
     def test_policy_cfn_notallowed(self):
-        pf = policy_path + 'notallowed.json'
-        self.m.StubOutWithMock(base_policy.Enforcer, '_get_policy_path')
-        base_policy.Enforcer._get_policy_path().MultipleTimes().AndReturn(pf)
-        self.m.ReplayAll()
+        self.stub_policyfile('notallowed.json')
 
         enforcer = policy.Enforcer(scope='cloudformation')
 
@@ -73,13 +74,9 @@ class TestPolicyEnforcer(HeatTestCase):
             # Everything should raise the default exception.Forbidden
             self.assertRaises(exception.Forbidden, enforcer.enforce, ctx,
                               action, {})
-        self.m.VerifyAll()
 
     def test_policy_cfn_deny_stack_user(self):
-        pf = policy_path + 'deny_stack_user.json'
-        self.m.StubOutWithMock(base_policy.Enforcer, '_get_policy_path')
-        base_policy.Enforcer._get_policy_path().MultipleTimes().AndReturn(pf)
-        self.m.ReplayAll()
+        self.stub_policyfile('deny_stack_user.json')
 
         enforcer = policy.Enforcer(scope='cloudformation')
 
@@ -87,31 +84,23 @@ class TestPolicyEnforcer(HeatTestCase):
         for action in self.cfn_actions:
             # Everything apart from DescribeStackResource should be Forbidden
             if action == "DescribeStackResource":
-                enforcer.enforce(ctx, action, {})
+                enforcer.enforce(ctx, action)
             else:
                 self.assertRaises(exception.Forbidden, enforcer.enforce, ctx,
                                   action, {})
-        self.m.VerifyAll()
 
     def test_policy_cfn_allow_non_stack_user(self):
-        pf = policy_path + 'deny_stack_user.json'
-        self.m.StubOutWithMock(base_policy.Enforcer, '_get_policy_path')
-        base_policy.Enforcer._get_policy_path().MultipleTimes().AndReturn(pf)
-        self.m.ReplayAll()
+        self.stub_policyfile('deny_stack_user.json')
 
         enforcer = policy.Enforcer(scope='cloudformation')
 
         ctx = utils.dummy_context(roles=['not_a_stack_user'])
         for action in self.cfn_actions:
             # Everything should be allowed
-            enforcer.enforce(ctx, action, {})
-        self.m.VerifyAll()
+            enforcer.enforce(ctx, action)
 
     def test_policy_cw_deny_stack_user(self):
-        pf = policy_path + 'deny_stack_user.json'
-        self.m.StubOutWithMock(base_policy.Enforcer, '_get_policy_path')
-        base_policy.Enforcer._get_policy_path().MultipleTimes().AndReturn(pf)
-        self.m.ReplayAll()
+        self.stub_policyfile('deny_stack_user.json')
 
         enforcer = policy.Enforcer(scope='cloudwatch')
 
@@ -119,54 +108,39 @@ class TestPolicyEnforcer(HeatTestCase):
         for action in self.cw_actions:
             # Everything apart from PutMetricData should be Forbidden
             if action == "PutMetricData":
-                enforcer.enforce(ctx, action, {})
+                enforcer.enforce(ctx, action)
             else:
                 self.assertRaises(exception.Forbidden, enforcer.enforce, ctx,
                                   action, {})
-        self.m.VerifyAll()
 
     def test_policy_cw_allow_non_stack_user(self):
-        pf = policy_path + 'deny_stack_user.json'
-        self.m.StubOutWithMock(base_policy.Enforcer, '_get_policy_path')
-        base_policy.Enforcer._get_policy_path().MultipleTimes().AndReturn(pf)
-        self.m.ReplayAll()
+        self.stub_policyfile('deny_stack_user.json')
 
         enforcer = policy.Enforcer(scope='cloudwatch')
 
         ctx = utils.dummy_context(roles=['not_a_stack_user'])
         for action in self.cw_actions:
             # Everything should be allowed
-            enforcer.enforce(ctx, action, {})
-        self.m.VerifyAll()
+            enforcer.enforce(ctx, action)
 
     def test_clear(self):
-        pf = policy_path + 'deny_stack_user.json'
-        self.m.StubOutWithMock(base_policy.Enforcer, '_get_policy_path')
-        base_policy.Enforcer._get_policy_path().MultipleTimes().AndReturn(pf)
-        self.m.ReplayAll()
+        self.stub_policyfile('deny_stack_user.json')
 
         enforcer = policy.Enforcer()
         enforcer.load_rules(force_reload=True)
         enforcer.clear()
-        self.assertEqual(enforcer.enforcer.rules, {})
-        self.m.VerifyAll()
+        self.assertEqual({}, enforcer.enforcer.rules)
 
     def test_set_rules_overwrite_true(self):
-        pf = policy_path + 'deny_stack_user.json'
-        self.m.StubOutWithMock(base_policy.Enforcer, '_get_policy_path')
-        base_policy.Enforcer._get_policy_path().MultipleTimes().AndReturn(pf)
-        self.m.ReplayAll()
+        self.stub_policyfile('deny_stack_user.json')
 
         enforcer = policy.Enforcer()
         enforcer.load_rules(True)
         enforcer.set_rules({'test_heat_rule': 1}, True)
-        self.assertEqual(enforcer.enforcer.rules, {'test_heat_rule': 1})
+        self.assertEqual({'test_heat_rule': 1}, enforcer.enforcer.rules)
 
     def test_set_rules_overwrite_false(self):
-        pf = policy_path + 'deny_stack_user.json'
-        self.m.StubOutWithMock(base_policy.Enforcer, '_get_policy_path')
-        base_policy.Enforcer._get_policy_path().MultipleTimes().AndReturn(pf)
-        self.m.ReplayAll()
+        self.stub_policyfile('deny_stack_user.json')
 
         enforcer = policy.Enforcer()
         enforcer.load_rules(True)
@@ -174,10 +148,7 @@ class TestPolicyEnforcer(HeatTestCase):
         self.assertIn('test_heat_rule', enforcer.enforcer.rules)
 
     def test_load_rules_force_reload_true(self):
-        pf = policy_path + 'deny_stack_user.json'
-        self.m.StubOutWithMock(base_policy.Enforcer, '_get_policy_path')
-        base_policy.Enforcer._get_policy_path().MultipleTimes().AndReturn(pf)
-        self.m.ReplayAll()
+        self.stub_policyfile('deny_stack_user.json')
 
         enforcer = policy.Enforcer()
         enforcer.set_rules({'test_heat_rule': 'test'})
@@ -185,10 +156,7 @@ class TestPolicyEnforcer(HeatTestCase):
         self.assertNotIn({'test_heat_rule': 'test'}, enforcer.enforcer.rules)
 
     def test_load_rules_force_reload_false(self):
-        pf = policy_path + 'deny_stack_user.json'
-        self.m.StubOutWithMock(base_policy.Enforcer, '_get_policy_path')
-        base_policy.Enforcer._get_policy_path().MultipleTimes().AndReturn(pf)
-        self.m.ReplayAll()
+        self.stub_policyfile('deny_stack_user.json')
 
         enforcer = policy.Enforcer()
         enforcer.load_rules(True)
@@ -197,15 +165,25 @@ class TestPolicyEnforcer(HeatTestCase):
         self.assertIn('test_heat_rule', enforcer.enforcer.rules)
 
     def test_default_rule(self):
-        pf = policy_path + 'deny_stack_user.json'
-        self.m.StubOutWithMock(base_policy.Enforcer, '_get_policy_path')
-        base_policy.Enforcer._get_policy_path().MultipleTimes().AndReturn(pf)
-        self.m.ReplayAll()
+        self.stub_policyfile('deny_stack_user.json')
 
         ctx = utils.dummy_context(roles=['not_a_stack_user'])
         default_rule = base_policy.FalseCheck()
         enforcer = policy.Enforcer(scope='cloudformation',
                                    exc=None, default_rule=default_rule)
         action = 'no_such_action'
-        self.assertEqual(enforcer.enforce(ctx, action, {}), False)
-        self.m.VerifyAll()
+        self.assertFalse(enforcer.enforce(ctx, action))
+
+    def test_check_admin(self):
+        self.stub_policyfile('check_admin.json')
+
+        enforcer = policy.Enforcer()
+
+        ctx = utils.dummy_context(roles=[])
+        self.assertFalse(enforcer.check_is_admin(ctx))
+
+        ctx = utils.dummy_context(roles=['not_admin'])
+        self.assertFalse(enforcer.check_is_admin(ctx))
+
+        ctx = utils.dummy_context(roles=['admin'])
+        self.assertTrue(enforcer.check_is_admin(ctx))
