@@ -301,6 +301,27 @@ class HOTemplateTest(HeatTestCase):
         err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.OUTPUTS)
         self.assertIn('Value', str(err))
 
+    def test_resource_group_list_join(self):
+        """Test list_join on a ResourceGroup's inner attributes
+
+        This should not fail during validation (i.e. before the ResourceGroup
+        can return the list of the runtime values.
+        """
+        hot_tpl = template_format.parse('''
+        heat_template_version: 2013-05-23
+        resources:
+          rg:
+            type: OS::Heat::ResourceGroup
+            properties:
+              count: 3
+              resource_def:
+                type: OS::Nova::Server
+        ''')
+        tmpl = parser.Template(hot_tpl)
+        stack = parser.Stack(utils.dummy_context(), 'test_stack', tmpl)
+        snippet = {'Fn::Join': ["\n", {'get_attr': ['rg', 'name']}]}
+        self.assertEqual(self.resolve(snippet, tmpl, stack), '')
+
     def test_str_replace(self):
         """Test str_replace function."""
 
@@ -1014,7 +1035,9 @@ class HOTParamValidatorTest(HeatTestCase):
         schema = param['db_name']
 
         def v(value):
-            hot_param.HOTParamSchema.from_dict(schema).validate(name, value)
+            param_schema = hot_param.HOTParamSchema.from_dict(schema)
+            param_schema.validate()
+            param_schema.validate_value(name, value)
             return True
 
         value = 'wp'
@@ -1063,7 +1086,7 @@ class HOTParamValidatorTest(HeatTestCase):
         def run_parameters(value):
             tmpl.parameters(
                 identifier.HeatIdentifier('', "stack_testit", None),
-                {'db_name': value})
+                {'db_name': value}).validate(validate_value=True)
             return True
 
         value = 'wp'
@@ -1103,7 +1126,9 @@ class HOTParamValidatorTest(HeatTestCase):
         schema = param['db_port']
 
         def v(value):
-            hot_param.HOTParamSchema.from_dict(schema).validate(name, value)
+            param_schema = hot_param.HOTParamSchema.from_dict(schema)
+            param_schema.validate()
+            param_schema.validate_value(name, value)
             return True
 
         value = 29999
@@ -1144,7 +1169,9 @@ class HOTParamValidatorTest(HeatTestCase):
         schema = param['param1']
 
         def v(value):
-            hot_param.HOTParamSchema.from_dict(schema).validate(name, value)
+            param_schema = hot_param.HOTParamSchema.from_dict(schema)
+            param_schema.validate()
+            param_schema.validate_value(name, value)
             return True
 
         value = "1"
@@ -1169,10 +1196,9 @@ class HOTParamValidatorTest(HeatTestCase):
                     {'range': {'min': 30000, 'max': 50000},
                      'description': range_desc}]}}
 
-        schema = param['db_port']
-
+        schema = hot_param.HOTParamSchema.from_dict(param['db_port'])
         err = self.assertRaises(constraints.InvalidSchemaError,
-                                hot_param.HOTParamSchema.from_dict, schema)
+                                schema.validate)
         self.assertIn(range_desc, str(err))
 
     def test_validate_schema_wrong_key(self):
