@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -13,24 +12,26 @@
 #    under the License.
 
 import copy
+import uuid
 
 import mox
+from neutronclient.v2_0 import client as neutronclient
 
-from heat.engine import environment
-from heat.tests.v1_1 import fakes
 from heat.common import exception
 from heat.common import template_format
+from heat.engine import clients
+from heat.engine import environment
 from heat.engine import parser
 from heat.engine import resource
-from heat.engine import scheduler
+from heat.engine.resources import image
 from heat.engine.resources import instance as instances
 from heat.engine.resources import network_interface
 from heat.engine.resources import nova_utils
+from heat.engine import scheduler
 from heat.openstack.common import uuidutils
 from heat.tests.common import HeatTestCase
 from heat.tests import utils
-
-from neutronclient.v2_0 import client as neutronclient
+from heat.tests.v1_1 import fakes
 
 
 wp_template = '''
@@ -70,7 +71,7 @@ class InstancesTest(HeatTestCase):
         template = parser.Template(t)
         stack = parser.Stack(utils.dummy_context(), stack_name, template,
                              environment.Environment({'KeyName': 'test'}),
-                             stack_id=uuidutils.generate_uuid())
+                             stack_id=str(uuid.uuid4()))
         return (t, stack)
 
     def _setup_test_instance(self, return_server, name, image_id=None,
@@ -86,15 +87,12 @@ class InstancesTest(HeatTestCase):
 
         self.m.StubOutWithMock(instance, 'nova')
         instance.nova().MultipleTimes().AndReturn(self.fc)
+        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
+        clients.OpenStackClients.nova().MultipleTimes().AndReturn(self.fc)
 
         instance.t = instance.stack.resolve_runtime_data(instance.t)
 
         if stub_create:
-            # need to resolve the template functions
-            server_userdata = nova_utils.build_userdata(
-                instance,
-                instance.t['Properties']['UserData'])
-            instance.mime_string = server_userdata
             self.m.StubOutWithMock(self.fc.servers, 'create')
             self.fc.servers.create(
                 image=1, flavor=1, key_name='test',
@@ -103,7 +101,7 @@ class InstancesTest(HeatTestCase):
                     instance.name,
                     limit=instance.physical_resource_name_limit),
                 security_groups=None,
-                userdata=server_userdata, scheduler_hints=None,
+                userdata=mox.IgnoreArg(), scheduler_hints=None,
                 meta=None, nics=None, availability_zone=None).AndReturn(
                     return_server)
 
@@ -124,10 +122,10 @@ class InstancesTest(HeatTestCase):
         self.assertTrue(instance.id > 0)
 
         expected_ip = return_server.networks['public'][0]
-        self.assertEqual(instance.FnGetAtt('PublicIp'), expected_ip)
-        self.assertEqual(instance.FnGetAtt('PrivateIp'), expected_ip)
-        self.assertEqual(instance.FnGetAtt('PrivateDnsName'), expected_ip)
-        self.assertEqual(instance.FnGetAtt('PrivateDnsName'), expected_ip)
+        self.assertEqual(expected_ip, instance.FnGetAtt('PublicIp'))
+        self.assertEqual(expected_ip, instance.FnGetAtt('PrivateIp'))
+        self.assertEqual(expected_ip, instance.FnGetAtt('PrivateDnsName'))
+        self.assertEqual(expected_ip, instance.FnGetAtt('PrivateDnsName'))
 
         self.m.VerifyAll()
 
@@ -137,7 +135,7 @@ class InstancesTest(HeatTestCase):
                                              'in_create_imgid',
                                              image_id='1')
         self.m.StubOutWithMock(uuidutils, "is_uuid_like")
-        uuidutils.is_uuid_like('1').AndReturn(True)
+        uuidutils.is_uuid_like('1').MultipleTimes().AndReturn(True)
 
         self.m.ReplayAll()
         scheduler.TaskRunner(instance.create)()
@@ -146,10 +144,10 @@ class InstancesTest(HeatTestCase):
         self.assertTrue(instance.id > 0)
 
         expected_ip = return_server.networks['public'][0]
-        self.assertEqual(instance.FnGetAtt('PublicIp'), expected_ip)
-        self.assertEqual(instance.FnGetAtt('PrivateIp'), expected_ip)
-        self.assertEqual(instance.FnGetAtt('PublicDnsName'), expected_ip)
-        self.assertEqual(instance.FnGetAtt('PrivateDnsName'), expected_ip)
+        self.assertEqual(expected_ip, instance.FnGetAtt('PublicIp'))
+        self.assertEqual(expected_ip, instance.FnGetAtt('PrivateIp'))
+        self.assertEqual(expected_ip, instance.FnGetAtt('PublicDnsName'))
+        self.assertEqual(expected_ip, instance.FnGetAtt('PrivateDnsName'))
 
         self.m.VerifyAll()
 
@@ -162,11 +160,11 @@ class InstancesTest(HeatTestCase):
         instance = instances.Instance('instance_create_image_err',
                                       t['Resources']['WebServer'], stack)
 
-        self.m.StubOutWithMock(instance, 'nova')
-        instance.nova().MultipleTimes().AndReturn(self.fc)
+        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
+        clients.OpenStackClients.nova().MultipleTimes().AndReturn(self.fc)
         self.m.ReplayAll()
 
-        self.assertRaises(exception.ImageNotFound, instance.handle_create)
+        self.assertRaises(ValueError, instance.handle_create)
 
         self.m.VerifyAll()
 
@@ -179,15 +177,15 @@ class InstancesTest(HeatTestCase):
         instance = instances.Instance('instance_create_image_err',
                                       t['Resources']['WebServer'], stack)
 
-        self.m.StubOutWithMock(instance, 'nova')
-        instance.nova().MultipleTimes().AndReturn(self.fc)
+        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
+        clients.OpenStackClients.nova().MultipleTimes().AndReturn(self.fc)
         self.m.StubOutWithMock(self.fc.client, "get_images_detail")
         self.fc.client.get_images_detail().AndReturn((
             200, {'images': [{'id': 1, 'name': 'CentOS 5.2'},
                              {'id': 4, 'name': 'CentOS 5.2'}]}))
         self.m.ReplayAll()
 
-        self.assertRaises(exception.NoUniqueImageFound, instance.handle_create)
+        self.assertRaises(ValueError, instance.handle_create)
 
         self.m.VerifyAll()
 
@@ -200,8 +198,8 @@ class InstancesTest(HeatTestCase):
         instance = instances.Instance('instance_create_image_err',
                                       t['Resources']['WebServer'], stack)
 
-        self.m.StubOutWithMock(instance, 'nova')
-        instance.nova().MultipleTimes().AndReturn(self.fc)
+        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
+        clients.OpenStackClients.nova().MultipleTimes().AndReturn(self.fc)
         self.m.StubOutWithMock(uuidutils, "is_uuid_like")
         uuidutils.is_uuid_like('1').AndReturn(True)
         self.m.StubOutWithMock(self.fc.client, "get_images_1")
@@ -209,7 +207,7 @@ class InstancesTest(HeatTestCase):
             instances.clients.novaclient.exceptions.NotFound(404))
         self.m.ReplayAll()
 
-        self.assertRaises(exception.ImageNotFound, instance.handle_create)
+        self.assertRaises(ValueError, instance.handle_create)
 
         self.m.VerifyAll()
 
@@ -257,15 +255,12 @@ class InstancesTest(HeatTestCase):
         return_server.get()
         self.m.ReplayAll()
 
-        try:
-            instance.check_create_complete(
-                (return_server, self.FakeVolumeAttach()))
-        except exception.Error as e:
-            self.assertEqual(
-                'Creation of server sample-server2 failed: Unknown (500)',
-                str(e))
-        else:
-            self.fail('Error not raised')
+        e = self.assertRaises(
+            exception.Error, instance.check_create_complete,
+            (return_server, self.FakeVolumeAttach()))
+        self.assertEqual(
+            'Creation of server sample-server2 failed: Unknown (500)',
+            str(e))
 
         self.m.VerifyAll()
 
@@ -278,14 +273,14 @@ class InstancesTest(HeatTestCase):
         instance = instances.Instance('instance_create_image_err',
                                       t['Resources']['WebServer'], stack)
 
-        self.m.StubOutWithMock(instance, 'nova')
-        instance.nova().MultipleTimes().AndReturn(self.fc)
+        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
+        clients.OpenStackClients.nova().MultipleTimes().AndReturn(self.fc)
 
         self.m.StubOutWithMock(uuidutils, "is_uuid_like")
-        uuidutils.is_uuid_like('1').AndReturn(True)
+        uuidutils.is_uuid_like('1').MultipleTimes().AndReturn(True)
         self.m.ReplayAll()
 
-        self.assertEqual(instance.validate(), None)
+        self.assertIsNone(instance.validate())
 
         self.m.VerifyAll()
 
@@ -304,8 +299,8 @@ class InstancesTest(HeatTestCase):
         mox.Replay(get)
 
         scheduler.TaskRunner(instance.delete)()
-        self.assertTrue(instance.resource_id is None)
-        self.assertEqual(instance.state, (instance.DELETE, instance.COMPLETE))
+        self.assertIsNone(instance.resource_id)
+        self.assertEqual((instance.DELETE, instance.COMPLETE), instance.state)
         self.m.VerifyAll()
 
     def test_instance_update_metadata(self):
@@ -316,7 +311,7 @@ class InstancesTest(HeatTestCase):
         update_template = copy.deepcopy(instance.t)
         update_template['Metadata'] = {'test': 123}
         scheduler.TaskRunner(instance.update, update_template)()
-        self.assertEqual(instance.metadata, {'test': 123})
+        self.assertEqual({'test': 123}, instance.metadata)
 
     def test_instance_update_instance_type(self):
         """
@@ -346,7 +341,7 @@ class InstancesTest(HeatTestCase):
         self.m.ReplayAll()
 
         scheduler.TaskRunner(instance.update, update_template)()
-        self.assertEqual(instance.state, (instance.UPDATE, instance.COMPLETE))
+        self.assertEqual((instance.UPDATE, instance.COMPLETE), instance.state)
         self.m.VerifyAll()
 
     def test_instance_update_instance_type_failed(self):
@@ -379,7 +374,7 @@ class InstancesTest(HeatTestCase):
         self.assertEqual(
             "Error: Resizing to 'm1.small' failed, status 'ACTIVE'",
             str(error))
-        self.assertEqual(instance.state, (instance.UPDATE, instance.FAILED))
+        self.assertEqual((instance.UPDATE, instance.FAILED), instance.state)
         self.m.VerifyAll()
 
     def test_instance_update_replace(self):
@@ -397,8 +392,13 @@ class InstancesTest(HeatTestCase):
         instance = self._create_test_instance(return_server,
                                               'in_update2')
 
+        self.m.StubOutWithMock(image.ImageConstraint, "validate")
+        image.ImageConstraint.validate(
+            mox.IgnoreArg(), mox.IgnoreArg()).MultipleTimes().AndReturn(True)
+        self.m.ReplayAll()
+
         update_template = copy.deepcopy(instance.t)
-        update_template['Properties']['KeyName'] = 'mustreplace'
+        update_template['Properties']['ImageId'] = 'mustreplace'
         updater = scheduler.TaskRunner(instance.update, update_template)
         self.assertRaises(resource.UpdateReplace, updater)
 
@@ -415,7 +415,7 @@ class InstancesTest(HeatTestCase):
         self.m.ReplayAll()
 
         scheduler.TaskRunner(instance.create)()
-        self.assertEqual(instance.state, (instance.CREATE, instance.COMPLETE))
+        self.assertEqual((instance.CREATE, instance.COMPLETE), instance.state)
 
     def test_instance_status_suspend_immediate(self):
         return_server = self.fc.servers.list()[1]
@@ -434,7 +434,7 @@ class InstancesTest(HeatTestCase):
         mox.Replay(get)
 
         scheduler.TaskRunner(instance.suspend)()
-        self.assertEqual(instance.state, (instance.SUSPEND, instance.COMPLETE))
+        self.assertEqual((instance.SUSPEND, instance.COMPLETE), instance.state)
 
         self.m.VerifyAll()
 
@@ -456,7 +456,7 @@ class InstancesTest(HeatTestCase):
         instance.state_set(instance.SUSPEND, instance.COMPLETE)
 
         scheduler.TaskRunner(instance.resume)()
-        self.assertEqual(instance.state, (instance.RESUME, instance.COMPLETE))
+        self.assertEqual((instance.RESUME, instance.COMPLETE), instance.state)
 
         self.m.VerifyAll()
 
@@ -482,7 +482,7 @@ class InstancesTest(HeatTestCase):
         self.m.ReplayAll()
 
         scheduler.TaskRunner(instance.suspend)()
-        self.assertEqual(instance.state, (instance.SUSPEND, instance.COMPLETE))
+        self.assertEqual((instance.SUSPEND, instance.COMPLETE), instance.state)
 
         self.m.VerifyAll()
 
@@ -510,7 +510,7 @@ class InstancesTest(HeatTestCase):
         instance.state_set(instance.SUSPEND, instance.COMPLETE)
 
         scheduler.TaskRunner(instance.resume)()
-        self.assertEqual(instance.state, (instance.RESUME, instance.COMPLETE))
+        self.assertEqual((instance.RESUME, instance.COMPLETE), instance.state)
 
         self.m.VerifyAll()
 
@@ -539,7 +539,7 @@ class InstancesTest(HeatTestCase):
         self.m.ReplayAll()
 
         scheduler.TaskRunner(instance.suspend)()
-        self.assertEqual(instance.state, (instance.SUSPEND, instance.COMPLETE))
+        self.assertEqual((instance.SUSPEND, instance.COMPLETE), instance.state)
 
         self.m.VerifyAll()
 
@@ -571,7 +571,41 @@ class InstancesTest(HeatTestCase):
         instance.state_set(instance.SUSPEND, instance.COMPLETE)
 
         scheduler.TaskRunner(instance.resume)()
-        self.assertEqual(instance.state, (instance.RESUME, instance.COMPLETE))
+        self.assertEqual((instance.RESUME, instance.COMPLETE), instance.state)
+
+        self.m.VerifyAll()
+
+    def test_instance_suspend_volumes_wait(self):
+        return_server = self.fc.servers.list()[1]
+        instance = self._create_test_instance(return_server,
+                                              'in_suspend_vol')
+
+        instance.resource_id = 1234
+        self.m.ReplayAll()
+
+        # Override the get_servers_1234 handler status to SUSPENDED, but keep
+        # it ACTIVE for the first two iterations of check_suspend_complete.
+        d1 = {'server': self.fc.client.get_servers_detail()[1]['servers'][0]}
+        d2 = copy.deepcopy(d1)
+        d1['server']['status'] = 'ACTIVE'
+        d2['server']['status'] = 'SUSPENDED'
+
+        # Return a dummy PollingTaskGroup to make check_suspend_complete step
+        def dummy_detach():
+            yield
+        dummy_tg = scheduler.PollingTaskGroup([dummy_detach, dummy_detach])
+        self.m.StubOutWithMock(instance, '_detach_volumes_task')
+        instance._detach_volumes_task().AndReturn(dummy_tg)
+
+        self.m.StubOutWithMock(self.fc.client, 'get_servers_1234')
+        get = self.fc.client.get_servers_1234
+        get().AndReturn((200, d1))
+        get().AndReturn((200, d1))
+        get().AndReturn((200, d2))
+        self.m.ReplayAll()
+
+        scheduler.TaskRunner(instance.suspend)()
+        self.assertEqual((instance.SUSPEND, instance.COMPLETE), instance.state)
 
         self.m.VerifyAll()
 
@@ -625,7 +659,7 @@ class InstancesTest(HeatTestCase):
         self.m.ReplayAll()
 
         scheduler.TaskRunner(instance.create)()
-        self.assertEqual(instance.state, (instance.CREATE, instance.COMPLETE))
+        self.assertEqual((instance.CREATE, instance.COMPLETE), instance.state)
 
         self.m.VerifyAll()
 
@@ -634,33 +668,34 @@ class InstancesTest(HeatTestCase):
         instance = self._create_test_instance(return_server,
                                               'build_nics')
 
-        self.assertEqual(None, instance._build_nics([]))
-        self.assertEqual(None, instance._build_nics(None))
+        self.assertIsNone(instance._build_nics([]))
+        self.assertIsNone(instance._build_nics(None))
         self.assertEqual([
             {'port-id': 'id3'}, {'port-id': 'id1'}, {'port-id': 'id2'}],
             instance._build_nics([
                 'id3', 'id1', 'id2']))
-        self.assertEqual([
-            {'port-id': 'id1'},
-            {'port-id': 'id2'},
-            {'port-id': 'id3'}], instance._build_nics([
+        self.assertEqual(
+            [{'port-id': 'id1'},
+             {'port-id': 'id2'},
+             {'port-id': 'id3'}],
+            instance._build_nics([
                 {'NetworkInterfaceId': 'id3', 'DeviceIndex': '3'},
                 {'NetworkInterfaceId': 'id1', 'DeviceIndex': '1'},
                 {'NetworkInterfaceId': 'id2', 'DeviceIndex': 2},
             ]))
-        self.assertEqual([
-            {'port-id': 'id1'},
-            {'port-id': 'id2'},
-            {'port-id': 'id3'},
-            {'port-id': 'id4'},
-            {'port-id': 'id5'}
-        ], instance._build_nics([
-            {'NetworkInterfaceId': 'id3', 'DeviceIndex': '3'},
-            {'NetworkInterfaceId': 'id1', 'DeviceIndex': '1'},
-            {'NetworkInterfaceId': 'id2', 'DeviceIndex': 2},
-            'id4',
-            'id5'
-        ]))
+        self.assertEqual(
+            [{'port-id': 'id1'},
+             {'port-id': 'id2'},
+             {'port-id': 'id3'},
+             {'port-id': 'id4'},
+             {'port-id': 'id5'}],
+            instance._build_nics([
+                {'NetworkInterfaceId': 'id3', 'DeviceIndex': '3'},
+                {'NetworkInterfaceId': 'id1', 'DeviceIndex': '1'},
+                {'NetworkInterfaceId': 'id2', 'DeviceIndex': 2},
+                'id4',
+                'id5']
+            ))
 
     def test_build_nics_with_security_groups(self):
         """
@@ -674,46 +709,59 @@ class InstancesTest(HeatTestCase):
         security_groups = ['security_group_1']
         self._test_security_groups(instance, security_groups)
 
-        security_groups = ['fake_id_1']
-        self._test_security_groups(instance, security_groups)
+        security_groups = ['0389f747-7785-4757-b7bb-2ab07e4b09c3']
+        self._test_security_groups(instance, security_groups, all_uuids=True)
 
-        security_groups = ['security_group_1', 'security_group_1']
-        self._test_security_groups(instance, security_groups)
+        security_groups = ['0389f747-7785-4757-b7bb-2ab07e4b09c3',
+                           '384ccd91-447c-4d83-832c-06974a7d3d05']
+        self._test_security_groups(instance, security_groups,
+                                   sg='two', all_uuids=True)
 
-        security_groups = ['fake_id_1', 'fake_id_1']
-        self._test_security_groups(instance, security_groups)
-
-        security_groups = ['security_group_1', 'fake_id_1']
-        self._test_security_groups(instance, security_groups)
-
-        security_groups = ['security_group_1', 'fake_id_2']
+        security_groups = ['security_group_1',
+                           '384ccd91-447c-4d83-832c-06974a7d3d05']
         self._test_security_groups(instance, security_groups, sg='two')
 
-        security_groups = ['wrong_group_id']
-        self._test_security_groups(instance, security_groups, sg='zero')
-
-        security_groups = ['wrong_group_id', 'fake_id_1']
-        self._test_security_groups(instance, security_groups)
-
         security_groups = ['wrong_group_name']
-        self._test_security_groups(instance, security_groups, sg='zero')
+        self._test_security_groups(
+            instance,
+            security_groups,
+            sg='zero',
+            get_secgroup_raises=exception.PhysicalResourceNotFound)
+
+        security_groups = ['wrong_group_name',
+                           '0389f747-7785-4757-b7bb-2ab07e4b09c3']
+        self._test_security_groups(
+            instance,
+            security_groups,
+            get_secgroup_raises=exception.PhysicalResourceNotFound)
 
         security_groups = ['wrong_group_name', 'security_group_1']
-        self._test_security_groups(instance, security_groups)
+        self._test_security_groups(
+            instance,
+            security_groups,
+            get_secgroup_raises=exception.PhysicalResourceNotFound)
 
-    def _test_security_groups(self, instance, security_groups, sg='one'):
+        security_groups = ['duplicate_group_name', 'security_group_1']
+        self._test_security_groups(
+            instance,
+            security_groups,
+            get_secgroup_raises=exception.PhysicalResourceNameAmbiguity)
+
+    def _test_security_groups(self, instance, security_groups, sg='one',
+                              all_uuids=False, get_secgroup_raises=None):
         fake_groups_list, props = self._get_fake_properties(sg)
 
-        def generate_sg_list():
-            yield fake_groups_list
-
         nclient = neutronclient.Client()
         self.m.StubOutWithMock(instance, 'neutron')
         instance.neutron().MultipleTimes().AndReturn(nclient)
 
-        self.m.StubOutWithMock(neutronclient.Client, 'list_security_groups')
-        neutronclient.Client.list_security_groups(
-            instance.resource_id).AndReturn(generate_sg_list())
+        if not all_uuids:
+            # list_security_groups only gets called when none of the requested
+            # groups look like UUIDs.
+            self.m.StubOutWithMock(
+                neutronclient.Client, 'list_security_groups')
+            neutronclient.Client.list_security_groups().AndReturn(
+                fake_groups_list)
 
         net_interface = network_interface.NetworkInterface
         self.m.StubOutWithMock(net_interface, 'network_id_from_subnet_id')
@@ -721,18 +769,24 @@ class InstancesTest(HeatTestCase):
             nclient,
             'fake_subnet_id').MultipleTimes().AndReturn('fake_network_id')
 
-        self.m.StubOutWithMock(neutronclient.Client, 'create_port')
-        neutronclient.Client.create_port(
-            {'port': props}).MultipleTimes().AndReturn(
-                {'port': {'id': 'fake_port_id'}})
+        if not get_secgroup_raises:
+            self.m.StubOutWithMock(neutronclient.Client, 'create_port')
+            neutronclient.Client.create_port(
+                {'port': props}).MultipleTimes().AndReturn(
+                    {'port': {'id': 'fake_port_id'}})
 
         self.m.ReplayAll()
 
-        self.assertEqual(
-            [{'port-id': 'fake_port_id'}],
-            instance._build_nics(None,
-                                 security_groups=security_groups,
-                                 subnet_id='fake_subnet_id'))
+        if get_secgroup_raises:
+            self.assertRaises(get_secgroup_raises, instance._build_nics, None,
+                              security_groups=security_groups,
+                              subnet_id='fake_subnet_id')
+        else:
+            self.assertEqual(
+                [{'port-id': 'fake_port_id'}],
+                instance._build_nics(None,
+                                     security_groups=security_groups,
+                                     subnet_id='fake_subnet_id'))
 
         self.m.VerifyAll()
         self.m.UnsetStubs()
@@ -741,16 +795,28 @@ class InstancesTest(HeatTestCase):
         fake_groups_list = {
             'security_groups': [
                 {
-                    'id': 'fake_id_1',
+                    'id': '0389f747-7785-4757-b7bb-2ab07e4b09c3',
                     'name': 'security_group_1',
                     'security_group_rules': [],
                     'description': 'no protocol'
                 },
                 {
-                    'id': 'fake_id_2',
+                    'id': '384ccd91-447c-4d83-832c-06974a7d3d05',
                     'name': 'security_group_2',
                     'security_group_rules': [],
                     'description': 'no protocol'
+                },
+                {
+                    'id': 'e91a0007-06a6-4a4a-8edb-1d91315eb0ef',
+                    'name': 'duplicate_group_name',
+                    'security_group_rules': [],
+                    'description': 'no protocol'
+                },
+                {
+                    'id': '8be37f3c-176d-4826-aa17-77d1d9df7b2e',
+                    'name': 'duplicate_group_name',
+                    'security_group_rules': [],
+                    'description': 'no protocol'
                 }
             ]
         }
@@ -760,15 +826,16 @@ class InstancesTest(HeatTestCase):
             'admin_state_up': True,
             'network_id': 'fake_network_id',
             'fixed_ips': [fixed_ip],
-            'security_groups': ['fake_id_1']
+            'security_groups': ['0389f747-7785-4757-b7bb-2ab07e4b09c3']
         }
 
         if sg == 'zero':
             props['security_groups'] = []
         elif sg == 'one':
-            props['security_groups'] = ['fake_id_1']
+            props['security_groups'] = ['0389f747-7785-4757-b7bb-2ab07e4b09c3']
         elif sg == 'two':
-            props['security_groups'] = ['fake_id_1', 'fake_id_2']
+            props['security_groups'] = ['0389f747-7785-4757-b7bb-2ab07e4b09c3',
+                                        '384ccd91-447c-4d83-832c-06974a7d3d05']
 
         return fake_groups_list, props
 
@@ -777,4 +844,51 @@ class InstancesTest(HeatTestCase):
         instance = self._create_test_instance(return_server,
                                               'wo_ipaddr')
 
-        self.assertEqual(instance.FnGetAtt('PrivateIp'), '0.0.0.0')
+        self.assertEqual('0.0.0.0', instance.FnGetAtt('PrivateIp'))
+
+    def test_default_instance_user(self):
+        """The default value for instance_user in heat.conf is ec2-user."""
+        return_server = self.fc.servers.list()[1]
+        instance = self._setup_test_instance(return_server, 'default_user')
+        self.m.StubOutWithMock(nova_utils, 'build_userdata')
+        nova_utils.build_userdata(instance, 'wordpress', 'ec2-user')
+        self.m.ReplayAll()
+        scheduler.TaskRunner(instance.create)()
+        self.m.VerifyAll()
+
+    def test_custom_instance_user(self):
+        """Test instance_user in heat.conf being set to a custom value.
+
+        Launching the instance should call build_userdata with the custom user
+        name.
+
+        This option is deprecated and will be removed in Juno.
+        """
+        return_server = self.fc.servers.list()[1]
+        instance = self._setup_test_instance(return_server, 'custom_user')
+        self.m.StubOutWithMock(instances.cfg.CONF, 'instance_user')
+        instances.cfg.CONF.instance_user = 'custom_user'
+        self.m.StubOutWithMock(nova_utils, 'build_userdata')
+        nova_utils.build_userdata(instance, 'wordpress', 'custom_user')
+        self.m.ReplayAll()
+        scheduler.TaskRunner(instance.create)()
+        self.m.VerifyAll()
+
+    def test_empty_instance_user(self):
+        """Test instance_user in heat.conf being empty.
+
+        Launching the instance should call build_userdata with
+        "ec2-user".
+
+        This behaviour is compatible with CloudFormation and will be
+        the default in Juno once the instance_user option gets removed.
+        """
+        return_server = self.fc.servers.list()[1]
+        instance = self._setup_test_instance(return_server, 'empty_user')
+        self.m.StubOutWithMock(instances.cfg.CONF, 'instance_user')
+        instances.cfg.CONF.instance_user = ''
+        self.m.StubOutWithMock(nova_utils, 'build_userdata')
+        nova_utils.build_userdata(instance, 'wordpress', 'ec2-user')
+        self.m.ReplayAll()
+        scheduler.TaskRunner(instance.create)()
+        self.m.VerifyAll()
