@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -12,18 +11,19 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import uuid
 
+from heat.common import template_format
+from heat.engine import clients
 from heat.engine import environment
-from heat.tests.v1_1 import fakes
+from heat.engine import parser
 from heat.engine.resources import instance as instances
 from heat.engine.resources import network_interface as network_interfaces
 from heat.engine.resources import nova_utils
-from heat.common import template_format
-from heat.engine import parser
 from heat.engine import scheduler
-from heat.openstack.common import uuidutils
 from heat.tests.common import HeatTestCase
 from heat.tests import utils
+from heat.tests.v1_1 import fakes
 
 
 wp_template = '''
@@ -159,7 +159,7 @@ class instancesTest(HeatTestCase):
                   'SubnetId': '4156c7a5-e8c4-4aff-a6e1-8f3c7bc83861'}
         stack = parser.Stack(utils.dummy_context(), stack_name, template,
                              environment.Environment(kwargs),
-                             stack_id=uuidutils.generate_uuid())
+                             stack_id=str(uuid.uuid4()))
 
         t['Resources']['WebServer']['Properties']['ImageId'] = 'CentOS 5.2'
         instance = instances.Instance('%s_name' % name,
@@ -167,6 +167,8 @@ class instancesTest(HeatTestCase):
 
         self.m.StubOutWithMock(instance, 'nova')
         instance.nova().MultipleTimes().AndReturn(self.fc)
+        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
+        clients.OpenStackClients.nova().MultipleTimes().AndReturn(self.fc)
 
         self.m.StubOutWithMock(instance, 'neutron')
         instance.neutron().MultipleTimes().AndReturn(FakeNeutron())
@@ -176,8 +178,14 @@ class instancesTest(HeatTestCase):
         # need to resolve the template functions
         server_userdata = nova_utils.build_userdata(
             instance,
-            instance.t['Properties']['UserData'])
-        instance.mime_string = server_userdata
+            instance.t['Properties']['UserData'],
+            'ec2-user')
+        self.m.StubOutWithMock(nova_utils, 'build_userdata')
+        nova_utils.build_userdata(
+            instance,
+            instance.t['Properties']['UserData'],
+            'ec2-user').AndReturn(server_userdata)
+
         self.m.StubOutWithMock(self.fc.servers, 'create')
         self.fc.servers.create(
             image=1, flavor=3, key_name='test',
@@ -201,7 +209,7 @@ class instancesTest(HeatTestCase):
                   'SubnetId': '4156c7a5-e8c4-4aff-a6e1-8f3c7bc83861'}
         stack = parser.Stack(utils.dummy_context(), stack_name, template,
                              environment.Environment(kwargs),
-                             stack_id=uuidutils.generate_uuid())
+                             stack_id=str(uuid.uuid4()))
 
         t['Resources']['WebServer']['Properties']['ImageId'] = 'CentOS 5.2'
 
@@ -217,6 +225,8 @@ class instancesTest(HeatTestCase):
 
         self.m.StubOutWithMock(instance, 'nova')
         instance.nova().MultipleTimes().AndReturn(self.fc)
+        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
+        clients.OpenStackClients.nova().MultipleTimes().AndReturn(self.fc)
 
         nic.t = nic.stack.resolve_runtime_data(nic.t)
         instance.t = instance.stack.resolve_runtime_data(instance.t)
@@ -224,8 +234,14 @@ class instancesTest(HeatTestCase):
         # need to resolve the template functions
         server_userdata = nova_utils.build_userdata(
             instance,
-            instance.t['Properties']['UserData'])
-        instance.mime_string = server_userdata
+            instance.t['Properties']['UserData'],
+            'ec2-user')
+        self.m.StubOutWithMock(nova_utils, 'build_userdata')
+        nova_utils.build_userdata(
+            instance,
+            instance.t['Properties']['UserData'],
+            'ec2-user').AndReturn(server_userdata)
+
         self.m.StubOutWithMock(self.fc.servers, 'create')
         self.fc.servers.create(
             image=1, flavor=3, key_name='test',
@@ -239,7 +255,7 @@ class instancesTest(HeatTestCase):
 
         # create network interface
         scheduler.TaskRunner(nic.create)()
-        stack.resources["nic1"] = nic
+        stack["nic1"] = nic
 
         scheduler.TaskRunner(instance.create)()
         return instance
@@ -252,10 +268,10 @@ class instancesTest(HeatTestCase):
         self.assertTrue(instance.id > 0)
 
         expected_ip = return_server.networks['public'][0]
-        self.assertEqual(instance.FnGetAtt('PublicIp'), expected_ip)
-        self.assertEqual(instance.FnGetAtt('PrivateIp'), expected_ip)
-        self.assertEqual(instance.FnGetAtt('PrivateDnsName'), expected_ip)
-        self.assertEqual(instance.FnGetAtt('PrivateDnsName'), expected_ip)
+        self.assertEqual(expected_ip, instance.FnGetAtt('PublicIp'))
+        self.assertEqual(expected_ip, instance.FnGetAtt('PrivateIp'))
+        self.assertEqual(expected_ip, instance.FnGetAtt('PrivateDnsName'))
+        self.assertEqual(expected_ip, instance.FnGetAtt('PrivateDnsName'))
 
         self.m.VerifyAll()
 
@@ -268,9 +284,9 @@ class instancesTest(HeatTestCase):
         self.assertTrue(instance.id > 0)
 
         expected_ip = return_server.networks['public'][0]
-        self.assertEqual(instance.FnGetAtt('PublicIp'), expected_ip)
-        self.assertEqual(instance.FnGetAtt('PrivateIp'), expected_ip)
-        self.assertEqual(instance.FnGetAtt('PrivateDnsName'), expected_ip)
-        self.assertEqual(instance.FnGetAtt('PrivateDnsName'), expected_ip)
+        self.assertEqual(expected_ip, instance.FnGetAtt('PublicIp'))
+        self.assertEqual(expected_ip, instance.FnGetAtt('PrivateIp'))
+        self.assertEqual(expected_ip, instance.FnGetAtt('PrivateDnsName'))
+        self.assertEqual(expected_ip, instance.FnGetAtt('PrivateDnsName'))
 
         self.m.VerifyAll()
