@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -14,22 +13,21 @@
 
 
 import mox
-
-import eventlet
-import unittest
-from nose.plugins.attrib import attr
-
 from oslo.config import cfg
-from heat.tests import fakes
-from heat.tests.utils import stack_delete_after
 
 from heat.common import identifier
 from heat.common import template_format
+from heat.engine import environment
 from heat.engine import parser
-from heat.engine import service
+from heat.engine.resources import image
 from heat.engine.resources import instance
-from heat.common import context
+from heat.engine.resources import nova_keypair
 from heat.engine.resources import wait_condition as wc
+from heat.engine import scheduler
+from heat.engine import service
+from heat.tests.common import HeatTestCase
+from heat.tests import fakes
+from heat.tests import utils
 
 
 test_template_metadata = '''
@@ -86,11 +84,11 @@ test_template_waitcondition = '''
     "KeyName" : {"Type" : "String", "Default": "mine" },
   },
   "Resources" : {
+    "WH" : {
+      "Type" : "AWS::CloudFormation::WaitConditionHandle"
+    },
     "S1": {
       "Type": "AWS::EC2::Instance",
-      "Metadata" : {
-        "test" : {"Fn::GetAtt": ["WC", "Data"]}
-      },
       "Properties": {
         "ImageId"      : "a",
         "InstanceType" : "m1.large",
@@ -101,9 +99,6 @@ test_template_waitcondition = '''
                                                 "\n" ] ] }
       }
     },
-    "WH" : {
-      "Type" : "AWS::CloudFormation::WaitConditionHandle"
-    },
     "WC" : {
       "Type" : "AWS::CloudFormation::WaitCondition",
       "DependsOn": "S1",
@@ -111,52 +106,65 @@ test_template_waitcondition = '''
         "Handle" : {"Ref" : "WH"},
         "Timeout" : "5"
       }
+    },
+    "S2": {
+      "Type": "AWS::EC2::Instance",
+      "Metadata" : {
+        "test" : {"Fn::GetAtt": ["WC", "Data"]}
+      },
+      "Properties": {
+        "ImageId"      : "a",
+        "InstanceType" : "m1.large",
+        "KeyName"      : { "Ref" : "KeyName" },
+        "UserData"     : "#!/bin/bash -v\n"
+      }
     }
   }
 }
 '''
 
 
-@attr(tag=['unit', 'resource', 'Metadata'])
-@attr(speed='slow')
-class MetadataRefreshTest(unittest.TestCase):
+class MetadataRefreshTest(HeatTestCase):
     '''
     The point of the test is to confirm that metadata gets updated
     when FnGetAtt() returns something different.
     gets called.
     '''
     def setUp(self):
-        self.m = mox.Mox()
-        self.m.StubOutWithMock(eventlet, 'sleep')
+        super(MetadataRefreshTest, self).setUp()
         self.fc = fakes.FakeKeystoneClient()
-
-    def tearDown(self):
-        self.m.UnsetStubs()
+        utils.setup_dummy_db()
 
     # Note tests creating a stack should be decorated with @stack_delete_after
     # to ensure the stack is properly cleaned up
     def create_stack(self, stack_name='test_stack', params={}):
         temp = template_format.parse(test_template_metadata)
         template = parser.Template(temp)
-        parameters = parser.Parameters(stack_name, template, params)
-        ctx = context.get_admin_context()
-        ctx.tenant_id = 'test_tenant'
-        stack = parser.Stack(ctx, stack_name, template, parameters,
+        ctx = utils.dummy_context()
+        stack = parser.Stack(ctx, stack_name, template,
+                             environment.Environment(params),
                              disable_rollback=True)
 
         self.stack_id = stack.store()
 
+        self.m.StubOutWithMock(nova_keypair.KeypairConstraint, 'validate')
+        nova_keypair.KeypairConstraint.validate(
+            mox.IgnoreArg(), mox.IgnoreArg()).MultipleTimes().AndReturn(True)
+        self.m.StubOutWithMock(image.ImageConstraint, 'validate')
+        image.ImageConstraint.validate(
+            mox.IgnoreArg(), mox.IgnoreArg()).MultipleTimes().AndReturn(True)
+
         self.m.StubOutWithMock(instance.Instance, 'handle_create')
-        self.m.StubOutWithMock(instance.Instance, 'check_active')
-        instance.Instance.handle_create().AndReturn(None)
-        instance.Instance.check_active().AndReturn(True)
-        instance.Instance.handle_create().AndReturn(None)
-        instance.Instance.check_active().AndReturn(True)
+        self.m.StubOutWithMock(instance.Instance, 'check_create_complete')
+        for cookie in (object(), object()):
+            instance.Instance.handle_create().AndReturn(cookie)
+            create_complete = instance.Instance.check_create_complete(cookie)
+            create_complete.InAnyOrder().AndReturn(True)
         self.m.StubOutWithMock(instance.Instance, 'FnGetAtt')
 
         return stack
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_FnGetAtt(self):
         self.stack = self.create_stack()
 
@@ -168,69 +176,77 @@ class MetadataRefreshTest(unittest.TestCase):
         self.m.ReplayAll()
         self.stack.create()
 
-        self.assertEqual(self.stack.state, self.stack.CREATE_COMPLETE)
+        self.assertEqual((self.stack.CREATE, self.stack.COMPLETE),
+                         self.stack.state)
 
-        s1 = self.stack.resources['S1']
-        s2 = self.stack.resources['S2']
+        s1 = self.stack['S1']
+        s2 = self.stack['S2']
         files = s1.metadata['AWS::CloudFormation::Init']['config']['files']
         cont = files['/tmp/random_file']['content']
-        self.assertEqual(s2.CREATE_COMPLETE, s2.state)
-        self.assertEqual(cont, 's2-ip=1.2.3.5')
+        self.assertEqual((s2.CREATE, s2.COMPLETE), s2.state)
+        self.assertEqual('s2-ip=1.2.3.5', cont)
 
         s1.metadata_update()
         s2.metadata_update()
         files = s1.metadata['AWS::CloudFormation::Init']['config']['files']
         cont = files['/tmp/random_file']['content']
-        self.assertEqual(cont, 's2-ip=10.0.0.5')
+        self.assertEqual('s2-ip=10.0.0.5', cont)
 
         self.m.VerifyAll()
 
 
-@attr(tag=['unit', 'resource', 'Metadata'])
-@attr(speed='slow')
-class WaitCondMetadataUpdateTest(unittest.TestCase):
+class WaitCondMetadataUpdateTest(HeatTestCase):
     def setUp(self):
-        self.m = mox.Mox()
-        self.ctx = context.get_admin_context()
-        self.ctx.tenant_id = 'test_tenant'
+        super(WaitCondMetadataUpdateTest, self).setUp()
+        utils.setup_dummy_db()
         self.fc = fakes.FakeKeystoneClient()
+
+        self.m.StubOutWithMock(service.EngineListener, 'start')
+        service.EngineListener.start().AndReturn(None)
+        self.m.ReplayAll()
         self.man = service.EngineService('a-host', 'a-topic')
         cfg.CONF.set_default('heat_waitcondition_server_url',
-                             'http://127.0.0.1:8000/v1/waitcondition')
-
-    def tearDown(self):
-        self.m.UnsetStubs()
+                             'http://server.test:8000/v1/waitcondition')
 
     # Note tests creating a stack should be decorated with @stack_delete_after
     # to ensure the stack is properly cleaned up
     def create_stack(self, stack_name='test_stack'):
         temp = template_format.parse(test_template_waitcondition)
         template = parser.Template(temp)
-        parameters = parser.Parameters(stack_name, template, {})
-        stack = parser.Stack(self.ctx, stack_name, template, parameters,
-                             disable_rollback=True)
+        ctx = utils.dummy_context()
+        stack = parser.Stack(ctx, stack_name, template, disable_rollback=True)
 
         self.stack_id = stack.store()
 
-        self.m.StubOutWithMock(instance.Instance, 'handle_create')
-        self.m.StubOutWithMock(instance.Instance, 'check_active')
+        self.m.StubOutWithMock(nova_keypair.KeypairConstraint, 'validate')
+        nova_keypair.KeypairConstraint.validate(
+            mox.IgnoreArg(), mox.IgnoreArg()).MultipleTimes().AndReturn(True)
+        self.m.StubOutWithMock(image.ImageConstraint, 'validate')
+        image.ImageConstraint.validate(
+            mox.IgnoreArg(), mox.IgnoreArg()).MultipleTimes().AndReturn(True)
 
-        instance.Instance.handle_create().AndReturn(None)
-        instance.Instance.check_active().AndReturn(True)
+        self.m.StubOutWithMock(instance.Instance, 'handle_create')
+        self.m.StubOutWithMock(instance.Instance, 'check_create_complete')
+        for cookie in (object(), object()):
+            instance.Instance.handle_create().AndReturn(cookie)
+            instance.Instance.check_create_complete(cookie).AndReturn(True)
 
         self.m.StubOutWithMock(wc.WaitConditionHandle, 'keystone')
         wc.WaitConditionHandle.keystone().MultipleTimes().AndReturn(self.fc)
 
-        id = identifier.ResourceIdentifier('test_tenant', stack.name,
+        id = identifier.ResourceIdentifier('test_tenant_id', stack.name,
                                            stack.id, '', 'WH')
         self.m.StubOutWithMock(wc.WaitConditionHandle, 'identifier')
         wc.WaitConditionHandle.identifier().MultipleTimes().AndReturn(id)
 
-        self.m.StubOutWithMock(eventlet, 'sleep')
+        self.m.StubOutWithMock(scheduler.TaskRunner, '_sleep')
+        self.m.StubOutWithMock(service.EngineService, 'load_user_creds')
+        service.EngineService.load_user_creds(
+            mox.IgnoreArg()).MultipleTimes().AndReturn(ctx)
 
         return stack
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_wait_meta(self):
         '''
         1 create stack
@@ -243,14 +259,14 @@ class WaitCondMetadataUpdateTest(unittest.TestCase):
         self.stack = self.create_stack()
 
         watch = self.stack['WC']
-        inst = self.stack['S1']
+        inst = self.stack['S2']
 
         def check_empty(sleep_time):
-            self.assertEqual(watch.FnGetAtt('Data'), '{}')
-            self.assertEqual(inst.metadata['test'], None)
+            self.assertEqual('{}', watch.FnGetAtt('Data'))
+            self.assertIsNone(inst.metadata['test'])
 
         def update_metadata(id, data, reason):
-            self.man.metadata_update(self.ctx,
+            self.man.metadata_update(utils.dummy_context(),
                                      dict(self.stack.identifier()),
                                      'WH',
                                      {'Data': data, 'Reason': reason,
@@ -259,21 +275,24 @@ class WaitCondMetadataUpdateTest(unittest.TestCase):
         def post_success(sleep_time):
             update_metadata('123', 'foo', 'bar')
 
-        eventlet.sleep(mox.IsA(int)).WithSideEffects(check_empty)
-        eventlet.sleep(mox.IsA(int)).WithSideEffects(post_success)
+        scheduler.TaskRunner._sleep(mox.IsA(int)).WithSideEffects(check_empty)
+        scheduler.TaskRunner._sleep(mox.IsA(int)).WithSideEffects(post_success)
+        scheduler.TaskRunner._sleep(mox.IsA(int)).MultipleTimes().AndReturn(
+            None)
 
         self.m.ReplayAll()
         self.stack.create()
 
-        self.assertEqual(self.stack.state, self.stack.CREATE_COMPLETE)
+        self.assertEqual((self.stack.CREATE, self.stack.COMPLETE),
+                         self.stack.state)
 
-        self.assertEqual(watch.FnGetAtt('Data'), '{"123": "foo"}')
-        self.assertEqual(inst.metadata['test'], '{"123": "foo"}')
+        self.assertEqual('{"123": "foo"}', watch.FnGetAtt('Data'))
+        self.assertEqual('{"123": "foo"}', inst.metadata['test'])
 
         update_metadata('456', 'blarg', 'wibble')
-        self.assertEqual(watch.FnGetAtt('Data'),
-                         '{"123": "foo", "456": "blarg"}')
-        self.assertEqual(inst.metadata['test'],
-                         '{"123": "foo", "456": "blarg"}')
+        self.assertEqual('{"123": "foo", "456": "blarg"}',
+                         watch.FnGetAtt('Data'))
+        self.assertEqual('{"123": "foo", "456": "blarg"}',
+                         inst.metadata['test'])
 
         self.m.VerifyAll()
