@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -14,6 +13,9 @@
 
 
 import copy
+import json
+import mock
+from requests import exceptions
 
 from oslo.config import cfg
 
@@ -26,9 +28,9 @@ from heat.db import api as db_api
 from heat.engine import parser
 from heat.engine import resource
 from heat.engine import scheduler
+from heat.tests.common import HeatTestCase
 from heat.tests import generic_resource as generic_rsrc
 from heat.tests import utils
-from heat.tests.common import HeatTestCase
 
 
 class NestedStackTest(HeatTestCase):
@@ -72,14 +74,21 @@ Outputs:
         t = template_format.parse(template)
         stack = self.parse_stack(t)
         stack.create()
-        self.assertEqual(stack.state, (stack.CREATE, stack.COMPLETE))
+        self.assertEqual((stack.CREATE, stack.COMPLETE), stack.state)
         return stack
 
-    def parse_stack(self, t):
+    def adopt_stack(self, template, adopt_data):
+        t = template_format.parse(template)
+        stack = self.parse_stack(t, adopt_data)
+        stack.adopt()
+        self.assertEqual((stack.ADOPT, stack.COMPLETE), stack.state)
+        return stack
+
+    def parse_stack(self, t, data=None):
         ctx = utils.dummy_context('test_username', 'aaaa', 'password')
         stack_name = 'test_stack'
         tmpl = parser.Template(t)
-        stack = parser.Stack(ctx, stack_name, tmpl)
+        stack = parser.Stack(ctx, stack_name, tmpl, adopt_stack_data=data)
         stack.store()
         return stack
 
@@ -91,7 +100,7 @@ Outputs:
         stack = self.create_stack(self.test_template)
         rsrc = stack['the_nested']
         nested_name = utils.PhysName(stack.name, 'the_nested')
-        self.assertEqual(nested_name, rsrc.physical_resource_name())
+        self.assertEqual(rsrc.physical_resource_name(), nested_name)
         arn_prefix = ('arn:openstack:heat::aaaa:stacks/%s/' %
                       rsrc.physical_resource_name())
         self.assertTrue(rsrc.FnGetRefId().startswith(arn_prefix))
@@ -109,6 +118,98 @@ Outputs:
 
         self.m.VerifyAll()
 
+    def test_nested_stack_adopt(self):
+        resource._register_class('GenericResource',
+                                 generic_rsrc.GenericResource)
+        urlfetch.get('https://server.test/the.template').MultipleTimes().\
+            AndReturn('''
+            HeatTemplateFormatVersion: '2012-12-12'
+            Parameters:
+              KeyName:
+                Type: String
+            Resources:
+              NestedResource:
+                Type: GenericResource
+            Outputs:
+              Foo:
+                Value: bar
+            ''')
+        self.m.ReplayAll()
+
+        adopt_data = {
+            "resources": {
+                "the_nested": {
+                    "resource_id": "test-res-id",
+                    "resources": {
+                        "NestedResource": {
+                            "resource_id": "test-nested-res-id"
+                        }
+                    }
+                }
+            }
+        }
+
+        stack = self.adopt_stack(self.test_template, adopt_data)
+        self.assertEqual((stack.ADOPT, stack.COMPLETE), stack.state)
+        rsrc = stack['the_nested']
+        self.assertEqual((rsrc.ADOPT, rsrc.COMPLETE), rsrc.state)
+        nested_name = utils.PhysName(stack.name, 'the_nested')
+        self.assertEqual(nested_name, rsrc.physical_resource_name())
+        self.assertEqual('test-nested-res-id',
+                         rsrc.nested()['NestedResource'].resource_id)
+        rsrc.delete()
+        self.m.VerifyAll()
+
+    def test_nested_stack_adopt_fail(self):
+        resource._register_class('GenericResource',
+                                 generic_rsrc.GenericResource)
+        urlfetch.get('https://server.test/the.template').MultipleTimes().\
+            AndReturn('''
+            HeatTemplateFormatVersion: '2012-12-12'
+            Parameters:
+              KeyName:
+                Type: String
+            Resources:
+              NestedResource:
+                Type: GenericResource
+            Outputs:
+              Foo:
+                Value: bar
+            ''')
+        self.m.ReplayAll()
+
+        adopt_data = {
+            "resources": {
+                "the_nested": {
+                    "resource_id": "test-res-id",
+                    "resources": {
+                    }
+                }
+            }
+        }
+
+        stack = self.adopt_stack(self.test_template, adopt_data)
+        rsrc = stack['the_nested']
+        self.assertEqual((rsrc.ADOPT, rsrc.FAILED), rsrc.nested().state)
+        nested_name = utils.PhysName(stack.name, 'the_nested')
+        self.assertEqual(nested_name, rsrc.physical_resource_name())
+        rsrc.delete()
+        self.m.VerifyAll()
+
+    def test_nested_stack_create_with_timeout(self):
+        urlfetch.get('https://server.test/the.template').MultipleTimes().\
+            AndReturn(self.nested_template)
+        self.m.ReplayAll()
+
+        timeout_template = template_format.parse(
+            copy.deepcopy(self.test_template))
+        props = timeout_template['Resources']['the_nested']['Properties']
+        props['TimeoutInMinutes'] = '50'
+
+        stack = self.create_stack(json.dumps(timeout_template))
+        self.assertEqual((stack.CREATE, stack.COMPLETE), stack.state)
+        self.m.VerifyAll()
+
     def test_nested_stack_create_exceeds_resource_limit(self):
         cfg.CONF.set_override('max_resources_per_stack', 1)
         resource._register_class('GenericResource',
@@ -131,7 +232,7 @@ Outputs:
         t = template_format.parse(self.test_template)
         stack = self.parse_stack(t)
         stack.create()
-        self.assertEquals(stack.state, (stack.CREATE, stack.FAILED))
+        self.assertEqual((stack.CREATE, stack.FAILED), stack.state)
         self.assertIn('Maximum resources per stack exceeded',
                       stack.status_reason)
 
@@ -159,9 +260,9 @@ Outputs:
         t = template_format.parse(self.test_template)
         stack = self.parse_stack(t)
         stack.create()
-        self.assertEquals(stack.state, (stack.CREATE, stack.COMPLETE))
+        self.assertEqual((stack.CREATE, stack.COMPLETE), stack.state)
         self.assertIn('NestedResource',
-                      stack.resources['the_nested'].nested().resources)
+                      stack['the_nested'].nested())
 
         self.m.VerifyAll()
 
@@ -184,7 +285,7 @@ Outputs:
         prop_diff = {'TemplateURL': 'https://server.test/new.template'}
         updater = rsrc.handle_update(new_res, {}, prop_diff)
         updater.run_to_completion()
-        self.assertEqual(True, rsrc.check_update_complete(updater))
+        self.assertIs(True, rsrc.check_update_complete(updater))
 
         # Expect the physical resource name staying the same after update,
         # so that the nested was actually updated instead of replaced.
@@ -232,7 +333,6 @@ Outputs:
 
         rsrc = stack['the_nested']
 
-        original_nested_id = rsrc.resource_id
         t = template_format.parse(self.test_template)
         new_res = copy.deepcopy(t['Resources']['the_nested'])
         new_res['Properties']['TemplateURL'] = (
@@ -240,7 +340,7 @@ Outputs:
         prop_diff = {'TemplateURL': 'https://server.test/new.template'}
         updater = rsrc.handle_update(new_res, {}, prop_diff)
         updater.run_to_completion()
-        self.assertEqual(True, rsrc.check_update_complete(updater))
+        self.assertIs(True, rsrc.check_update_complete(updater))
 
         self.assertEqual('foo', rsrc.FnGetAtt('Outputs.Bar'))
 
@@ -274,7 +374,6 @@ Outputs:
 
         rsrc = stack['the_nested']
 
-        original_nested_id = rsrc.resource_id
         t = template_format.parse(self.test_template)
         new_res = copy.deepcopy(t['Resources']['the_nested'])
         new_res['Properties']['TemplateURL'] = (
@@ -282,7 +381,7 @@ Outputs:
         prop_diff = {'TemplateURL': 'https://server.test/new.template'}
         ex = self.assertRaises(exception.RequestLimitExceeded,
                                rsrc.handle_update, new_res, {}, prop_diff)
-        self.assertIn(exception.StackResourceLimitExceeded.message,
+        self.assertIn(exception.StackResourceLimitExceeded.msg_fmt,
                       str(ex))
         rsrc.delete()
 
@@ -297,17 +396,17 @@ Outputs:
         rsrc = stack['the_nested']
 
         scheduler.TaskRunner(rsrc.suspend)()
-        self.assertEqual(rsrc.state, (rsrc.SUSPEND, rsrc.COMPLETE))
+        self.assertEqual((rsrc.SUSPEND, rsrc.COMPLETE), rsrc.state)
 
         scheduler.TaskRunner(rsrc.resume)()
-        self.assertEqual(rsrc.state, (rsrc.RESUME, rsrc.COMPLETE))
+        self.assertEqual((rsrc.RESUME, rsrc.COMPLETE), rsrc.state)
 
         rsrc.delete()
         self.m.VerifyAll()
 
     def test_nested_stack_three_deep(self):
         root_template = '''
-HeatTemplateFormat: 2012-12-12
+HeatTemplateFormatVersion: 2012-12-12
 Resources:
     Nested:
         Type: AWS::CloudFormation::Stack
@@ -315,7 +414,7 @@ Resources:
             TemplateURL: 'https://server.test/depth1.template'
 '''
         depth1_template = '''
-HeatTemplateFormat: 2012-12-12
+HeatTemplateFormatVersion: 2012-12-12
 Resources:
     Nested:
         Type: AWS::CloudFormation::Stack
@@ -323,7 +422,7 @@ Resources:
             TemplateURL: 'https://server.test/depth2.template'
 '''
         depth2_template = '''
-HeatTemplateFormat: 2012-12-12
+HeatTemplateFormatVersion: 2012-12-12
 Resources:
     Nested:
         Type: AWS::CloudFormation::Stack
@@ -347,7 +446,7 @@ Resources:
 
     def test_nested_stack_four_deep(self):
         root_template = '''
-HeatTemplateFormat: 2012-12-12
+HeatTemplateFormatVersion: 2012-12-12
 Resources:
     Nested:
         Type: AWS::CloudFormation::Stack
@@ -355,7 +454,7 @@ Resources:
             TemplateURL: 'https://server.test/depth1.template'
 '''
         depth1_template = '''
-HeatTemplateFormat: 2012-12-12
+HeatTemplateFormatVersion: 2012-12-12
 Resources:
     Nested:
         Type: AWS::CloudFormation::Stack
@@ -363,7 +462,7 @@ Resources:
             TemplateURL: 'https://server.test/depth2.template'
 '''
         depth2_template = '''
-HeatTemplateFormat: 2012-12-12
+HeatTemplateFormatVersion: 2012-12-12
 Resources:
     Nested:
         Type: AWS::CloudFormation::Stack
@@ -371,7 +470,7 @@ Resources:
             TemplateURL: 'https://server.test/depth3.template'
 '''
         depth3_template = '''
-HeatTemplateFormat: 2012-12-12
+HeatTemplateFormatVersion: 2012-12-12
 Resources:
     Nested:
         Type: AWS::CloudFormation::Stack
@@ -402,7 +501,7 @@ Resources:
 
     def test_nested_stack_four_wide(self):
         root_template = '''
-HeatTemplateFormat: 2012-12-12
+HeatTemplateFormatVersion: 2012-12-12
 Resources:
     Nested:
         Type: AWS::CloudFormation::Stack
@@ -447,7 +546,7 @@ Resources:
 
     def test_nested_stack_infinite_recursion(self):
         template = '''
-HeatTemplateFormat: 2012-12-12
+HeatTemplateFormatVersion: 2012-12-12
 Resources:
     Nested:
         Type: AWS::CloudFormation::Stack
@@ -461,10 +560,81 @@ Resources:
         t = template_format.parse(template)
         stack = self.parse_stack(t)
         stack.create()
-        self.assertEqual(stack.state, (stack.CREATE, stack.FAILED))
+        self.assertEqual((stack.CREATE, stack.FAILED), stack.state)
         self.assertIn('Recursion depth exceeds', stack.status_reason)
         self.m.VerifyAll()
 
+    def test_nested_stack_delete(self):
+        urlfetch.get('https://server.test/the.template').MultipleTimes().\
+            AndReturn(self.nested_template)
+        self.m.ReplayAll()
+
+        stack = self.create_stack(self.test_template)
+        rsrc = stack['the_nested']
+        scheduler.TaskRunner(rsrc.delete)()
+        self.assertEqual((stack.DELETE, stack.COMPLETE), rsrc.state)
+
+        nested_stack = parser.Stack.load(utils.dummy_context(
+            'test_username', 'aaaa', 'password'), rsrc.resource_id)
+        self.assertEqual((stack.DELETE, stack.COMPLETE), nested_stack.state)
+
+        self.m.VerifyAll()
+
+    def test_nested_stack_delete_then_delete_parent_stack(self):
+        urlfetch.get('https://server.test/the.template').MultipleTimes().\
+            AndReturn(self.nested_template)
+        self.m.ReplayAll()
+
+        stack = self.create_stack(self.test_template)
+        rsrc = stack['the_nested']
+
+        nested_stack = parser.Stack.load(utils.dummy_context(
+            'test_username', 'aaaa', 'password'), rsrc.resource_id)
+        nested_stack.delete()
+
+        stack = parser.Stack.load(utils.dummy_context(
+            'test_username', 'aaaa', 'password'), stack.id)
+        stack.delete()
+        self.assertEqual((stack.DELETE, stack.COMPLETE), stack.state)
+
+        self.m.VerifyAll()
+
+    def test_child_params(self):
+        t = template_format.parse(self.test_template)
+        stack = self.parse_stack(t)
+        nested_stack = stack['the_nested']
+        nested_stack.properties.data[nested_stack.PARAMETERS] = {'foo': 'bar'}
+
+        self.assertEqual({'foo': 'bar'}, nested_stack.child_params())
+
+    @mock.patch.object(urlfetch, 'get')
+    def test_child_template_when_file_is_fetched(self, mock_get):
+        mock_get.return_value = 'template_file'
+        t = template_format.parse(self.test_template)
+        stack = self.parse_stack(t)
+        nested_stack = stack['the_nested']
+
+        with mock.patch('heat.common.template_format.parse') as mock_parse:
+            mock_parse.return_value = 'child_template'
+            self.assertEqual('child_template', nested_stack.child_template())
+            mock_parse.assert_called_once_with('template_file')
+
+    @mock.patch.object(urlfetch, 'get')
+    def test_child_template_when_fetching_file_fails(self, mock_get):
+        mock_get.side_effect = exceptions.RequestException()
+        t = template_format.parse(self.test_template)
+        stack = self.parse_stack(t)
+        nested_stack = stack['the_nested']
+        self.assertRaises(ValueError, nested_stack.child_template)
+
+    @mock.patch.object(urlfetch, 'get')
+    def test_child_template_when_io_error(self, mock_get):
+        mock_get.side_effect = IOError()
+        t = template_format.parse(self.test_template)
+        stack = self.parse_stack(t)
+        nested_stack = stack['the_nested']
+        self.assertRaises(ValueError, nested_stack.child_template)
+
 
 class ResDataResource(generic_rsrc.GenericResource):
     def handle_create(self):
@@ -487,8 +657,8 @@ Outputs:
 '''
 
     def setUp(self):
-        resource._register_class("res.data.resource", ResDataResource)
         super(ResDataNestedStackTest, self).setUp()
+        resource._register_class("res.data.resource", ResDataResource)
 
     def test_res_data_delete(self):
         urlfetch.get('https://server.test/the.template').AndReturn(
@@ -497,6 +667,6 @@ Outputs:
         stack = self.create_stack(self.test_template)
         res = stack['the_nested'].nested()['nested_res']
         stack.delete()
-        self.assertEqual(stack.state, (stack.DELETE, stack.COMPLETE))
+        self.assertEqual((stack.DELETE, stack.COMPLETE), stack.state)
         self.assertRaises(exception.NotFound, db_api.resource_data_get, res,
                           'test')
