@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -12,13 +11,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from heat.tests.v1_1 import fakes
+from heat.common import template_format
+from heat.engine import clients
 from heat.engine.resources import instance as instances
 from heat.engine.resources import nova_utils
-from heat.common import template_format
 from heat.engine import scheduler
 from heat.tests.common import HeatTestCase
 from heat.tests import utils
+from heat.tests.v1_1 import fakes
 
 
 nokey_template = '''
@@ -60,14 +60,22 @@ class nokeyTest(HeatTestCase):
 
         self.m.StubOutWithMock(instance, 'nova')
         instance.nova().MultipleTimes().AndReturn(self.fc)
+        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
+        clients.OpenStackClients.nova().MultipleTimes().AndReturn(self.fc)
 
         instance.t = instance.stack.resolve_runtime_data(instance.t)
 
         # need to resolve the template functions
         server_userdata = nova_utils.build_userdata(
             instance,
-            instance.t['Properties']['UserData'])
-        instance.mime_string = server_userdata
+            instance.t['Properties']['UserData'],
+            'ec2-user')
+        self.m.StubOutWithMock(nova_utils, 'build_userdata')
+        nova_utils.build_userdata(
+            instance,
+            instance.t['Properties']['UserData'],
+            'ec2-user').AndReturn(server_userdata)
+
         self.m.StubOutWithMock(self.fc.servers, 'create')
         self.fc.servers.create(
             image=1, flavor=1, key_name=None,
