@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -15,40 +14,41 @@
 
 import pkgutil
 import sys
+
 import testtools
 
-import heat.engine
 from heat.common import plugin_loader
+import heat.engine
 
 
 class PluginLoaderTest(testtools.TestCase):
     def test_module_name(self):
-        self.assertEqual(plugin_loader._module_name('foo.bar', 'blarg.wibble'),
-                         'foo.bar.blarg.wibble')
+        self.assertEqual('foo.bar.blarg.wibble',
+                         plugin_loader._module_name('foo.bar', 'blarg.wibble'))
 
     def test_create_subpackage_single_path(self):
         pkg_name = 'heat.engine.test_single_path'
-        self.assertFalse(pkg_name in sys.modules)
+        self.assertNotIn(pkg_name, sys.modules)
         pkg = plugin_loader.create_subpackage('/tmp',
                                               'heat.engine',
                                               'test_single_path')
-        self.assertTrue(pkg_name in sys.modules)
+        self.assertIn(pkg_name, sys.modules)
         self.assertEqual(sys.modules[pkg_name], pkg)
-        self.assertEqual(pkg.__path__, ['/tmp'])
-        self.assertEqual(pkg.__name__, pkg_name)
+        self.assertEqual(['/tmp'], pkg.__path__)
+        self.assertEqual(pkg_name, pkg.__name__)
 
     def test_create_subpackage_path_list(self):
         path_list = ['/tmp']
         pkg_name = 'heat.engine.test_path_list'
-        self.assertFalse(pkg_name in sys.modules)
+        self.assertNotIn(pkg_name, sys.modules)
         pkg = plugin_loader.create_subpackage('/tmp',
                                               'heat.engine',
                                               'test_path_list')
-        self.assertTrue(pkg_name in sys.modules)
+        self.assertIn(pkg_name, sys.modules)
         self.assertEqual(sys.modules[pkg_name], pkg)
-        self.assertEqual(pkg.__path__, path_list)
-        self.assertFalse(pkg.__path__ is path_list)
-        self.assertEqual(pkg.__name__, pkg_name)
+        self.assertEqual(path_list, pkg.__path__)
+        self.assertNotIn(pkg.__path__, path_list)
+        self.assertEqual(pkg_name, pkg.__name__)
 
     def test_import_module_existing(self):
         import heat.engine.service
@@ -57,11 +57,10 @@ class PluginLoaderTest(testtools.TestCase):
         loaded = plugin_loader._import_module(importer,
                                               'heat.engine.service',
                                               heat.engine)
-        self.assertTrue(loaded is existing)
+        self.assertIs(existing, loaded)
 
     def test_import_module_garbage(self):
         importer = pkgutil.ImpImporter(heat.engine.__path__[0])
-        self.assertEqual(plugin_loader._import_module(importer,
-                                                      'wibble',
-                                                      heat.engine),
-                         None)
+        self.assertIsNone(plugin_loader._import_module(importer,
+                                                       'wibble',
+                                                       heat.engine))
