@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -17,14 +16,14 @@ from testtools import skipIf
 
 from heat.common import exception
 from heat.common import template_format
-from heat.openstack.common.importutils import try_import
-from heat.engine.resources import s3
-from heat.engine import resource
 from heat.engine import clients
+from heat.engine import resource
+from heat.engine.resources import s3
 from heat.engine import scheduler
+from heat.openstack.common.importutils import try_import
 from heat.tests.common import HeatTestCase
-from heat.tests import utils
 from heat.tests import fakes
+from heat.tests import utils
 
 swiftclient = try_import('swiftclient.client')
 
@@ -55,6 +54,13 @@ swift_template = '''
       "Properties" : {
         "AccessControl" : "Private"
       }
+    },
+    "S3Bucket_with_tags" : {
+      "Type" : "AWS::S3::Bucket",
+      "Properties" : {
+        "Tags" : [{"Key": "greeting", "Value": "hello"},
+                  {"Key": "location", "Value": "here"}]
+      }
     }
   }
 }
@@ -107,11 +113,8 @@ class s3Test(HeatTestCase):
 
         self.assertEqual(url, rsrc.FnGetAtt('WebsiteURL'))
 
-        try:
-            rsrc.FnGetAtt('Foo')
-            raise Exception('Expected InvalidTemplateAttribute')
-        except exception.InvalidTemplateAttribute:
-            pass
+        self.assertRaises(exception.InvalidTemplateAttribute,
+                          rsrc.FnGetAtt, 'Foo')
 
         self.assertRaises(resource.UpdateReplace,
                           rsrc.handle_update, {}, {}, {})
@@ -139,6 +142,26 @@ class s3Test(HeatTestCase):
         scheduler.TaskRunner(rsrc.delete)()
         self.m.VerifyAll()
 
+    def test_tags(self):
+        clients.OpenStackClients.keystone().AndReturn(
+            fakes.FakeKeystoneClient())
+        container_name = utils.PhysName('test_stack', 'test_resource')
+        swiftclient.Connection.put_container(
+            utils.PhysName('test_stack', 'test_resource'),
+            {'X-Container-Write': 'test_tenant:test_username',
+             'X-Container-Read': 'test_tenant:test_username',
+             'X-Container-Meta-S3-Tag-greeting': 'hello',
+             'X-Container-Meta-S3-Tag-location': 'here'}).AndReturn(None)
+        swiftclient.Connection.delete_container(
+            container_name).AndReturn(None)
+
+        self.m.ReplayAll()
+        t = template_format.parse(swift_template)
+        stack = utils.parse_stack(t)
+        rsrc = self.create_resource(t, stack, 'S3Bucket_with_tags')
+        scheduler.TaskRunner(rsrc.delete)()
+        self.m.VerifyAll()
+
     def test_public_read_write(self):
         clients.OpenStackClients.keystone().AndReturn(
             fakes.FakeKeystoneClient())
