@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -18,16 +17,18 @@ import string
 import sys
 import uuid
 
+from oslo.config import cfg
 import sqlalchemy
 
 from heat.common import context
 from heat.common import exception
+from heat.db import api as db_api
 from heat.engine import environment
 from heat.engine import parser
 from heat.engine import resource
+from heat.openstack.common.db.sqlalchemy import session
 
-from heat.db.sqlalchemy.session import get_engine
-from heat.db import migration
+get_engine = session.get_engine
 
 
 class UUIDStub(object):
@@ -113,7 +114,9 @@ def wr_delete_after(test_fn):
 
 
 def setup_dummy_db():
-    migration.db_sync()
+    cfg.CONF.set_default('sqlite_synchronous', False)
+    session.set_defaults(sql_connection="sqlite://", sqlite_db='heat.db')
+    db_api.db_sync()
     engine = get_engine()
     engine.connect()
 
@@ -130,23 +133,27 @@ def reset_dummy_db():
 
 
 def dummy_context(user='test_username', tenant_id='test_tenant_id',
-                  password='password', roles=[]):
+                  password='password', roles=[], user_id=None):
     return context.RequestContext.from_dict({
         'tenant_id': tenant_id,
         'tenant': 'test_tenant',
         'username': user,
+        'user_id': user_id,
         'password': password,
         'roles': roles,
+        'is_admin': False,
         'auth_url': 'http://server.test:5000/v2.0',
         'auth_token': 'abcd1234'
     })
 
 
-def parse_stack(t, params={}, stack_name='test_stack', stack_id=None):
+def parse_stack(t, params={}, stack_name='test_stack', stack_id=None,
+                timeout_mins=None):
     ctx = dummy_context()
     template = parser.Template(t)
     stack = parser.Stack(ctx, stack_name, template,
-                         environment.Environment(params), stack_id)
+                         environment.Environment(params), stack_id,
+                         timeout_mins=timeout_mins)
     stack.store()
 
     return stack
@@ -157,9 +164,12 @@ class PhysName(object):
     mock_short_id = 'x' * 12
 
     def __init__(self, stack_name, resource_name, limit=255):
-        self.stack_name = stack_name
-        self.resource_name = resource_name
-        self.limit = limit
+        name = '%s-%s-%s' % (stack_name,
+                             resource_name,
+                             self.mock_short_id)
+        self._physname = resource.Resource.reduce_physical_resource_name(
+            name, limit)
+        self.stack, self.res, self.sid = self._physname.rsplit('-', 2)
 
     def __eq__(self, physical_name):
         try:
@@ -171,14 +181,10 @@ class PhysName(object):
             return False
 
         # ignore the stack portion of the name, as it may have been truncated
-        return self.resource_name == res
+        return res == self.res
 
     def __ne__(self, physical_name):
         return not self.__eq__(physical_name)
 
     def __repr__(self):
-        name = '%s-%s-%s' % (self.stack_name,
-                             self.resource_name,
-                             self.mock_short_id)
-        return resource.Resource.reduce_physical_resource_name(
-            name, self.limit)
+        return self._physname
