@@ -14,14 +14,24 @@
 # limitations under the License.
 
 import httplib2
+import mock
 
 from novaclient import client as base_client
+from novaclient import exceptions as nova_exceptions
 from novaclient.v1_1 import client
 
 from heat.openstack.common.py3kcompat import urlutils
 from heat.tests import fakes
 
 
+def fake_exception(status_code=404, message=None, details=None):
+    resp = mock.Mock()
+    resp.status_code = status_code
+    resp.headers = None
+    body = {'error': {'message': message, 'details': details}}
+    return nova_exceptions.from_response(resp, body, None)
+
+
 class FakeClient(fakes.FakeClient, client.Client):
 
     def __init__(self, *args, **kwargs):
